<?xml version="1.0" encoding="utf-8" ?>
<!-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ -->
<!--                                                                                        -->
<!--                     Genesys Archive Control Stream Style Sheet                         -->
<!--                    Copyright (c) 2006-2007 Genesys Conferencing                        -->
<!--                                                                                        -->
<!-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ -->

<!DOCTYPE meetingarchive [<!ENTITY nbsp "&#160;">]>

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:meetingarchive="http://www.genesys.com/genmc/meetingarchive">
    <xsl:strip-space elements="metadata recording" />
    <xsl:output cdata-section-elements="event" />
    <xsl:output method="html" indent="no" encoding="utf-8" doctype-public="-//W3C//DTD HTML 4.01//EN" doctype-system="http://www.w3.org/TR/html4/strict.dtd" />

    <xsl:variable name="start_ts" select="number(/meetingarchive/recording/start/attribute::ts)" />
    
<xsl:template match="meetingarchive">

<html>
<head>
<title>Archive #<xsl:value-of select="metadata/archive/attribute::id" /> (<xsl:value-of select="metadata/archive/attribute::name"/>)</title>
<style type="text/css"><![CDATA[
body {font: 10pt Verdana, sans-serif; margin: 0; padding: 5px 10px 1em;}
h1 {font-size: 14pt;}
h2 {font-size: 12pt;}
div.element {margin: .5em 0; padding: 0;}
div.metadata {border: 1px solid #CCC; margin-bottom: 1em; background-color: #F9F9F9;}
div.recording {}
div.event-data {border: 1px solid #CCC; margin: 5px auto; padding: 2px; background-color: #F9F9F9;}
span.elem-name {font-weight: bold;}
]]>
</style>
<script type="text/javascript"><![CDATA[
function on_page_load()
{
    var divs = document.getElementsByTagName("div");
    var text;

    for(var index = 0; divs && index < divs.length; index++) {
        if(divs[index].className == "event-data") {
            text = new String();
            while(divs[index].firstChild) {
                text += decodeURIComponent(divs[index].firstChild.nodeValue);
                divs[index].removeChild(divs[index].firstChild);
            }
            text = text.replace(/>/g, ">\r\n");
            divs[index].appendChild(document.createTextNode(text));
        }
    }
}
]]>
</script>
</head>

<body onload="on_page_load()">
<h1>Archive #<xsl:value-of select="metadata/archive/attribute::id" /></h1>
<h2><xsl:value-of select="metadata/archive/attribute::name"/></h2>
<xsl:apply-templates select="*"/>
</body>
</html>

</xsl:template>

<xsl:template match="*">
<div><xsl:attribute name="class">element <xsl:value-of select="local-name(.)"/></xsl:attribute>
&lt;<span class="elem-name"><xsl:value-of select="local-name(.)"/></span><xsl:apply-templates select="attribute::*"/>&gt;
<xsl:apply-templates select="*"/>
<xsl:apply-templates select="text()"/>
&lt;/<span class="elem-name"><xsl:value-of select="local-name(.)"/></span>&gt;</div>
</xsl:template>

<!-- print attributes, each on a separate line, except the first one -->
<xsl:template match="attribute::*">&nbsp;
<xsl:if test="position() != 1"><br/>&nbsp;&nbsp;&nbsp;&nbsp;</xsl:if>
<xsl:choose>
<!-- process timestamps -->
<xsl:when test="local-name(.) = &quot;ts&quot;">
ts=&quot;<xsl:choose>
<xsl:when test="local-name(parent::node()) = &quot;start&quot; or local-name(parent::node()) = &quot;end&quot;">
<xsl:value-of select="round(((self::node() - 116444736000000000)) div 10000)"/>
</xsl:when>
<xsl:otherwise>
<xsl:value-of select="round(((self::node() - $start_ts)) div 10000)"/>
</xsl:otherwise>
</xsl:choose>&quot;
</xsl:when>
<xsl:otherwise>
<!-- other attributes -->
<xsl:value-of select="local-name(.)"/>=&quot;<xsl:value-of select="."/>&quot;
</xsl:otherwise>
</xsl:choose>
</xsl:template>

<xsl:template match="event/text()">
<pre><div class="event-data"><xsl:value-of select="."/></div></pre>
</xsl:template>

</xsl:stylesheet>
