/**sfh
******************************************************************************

	appshareplayer
	~~~~~~~~~~~~~~
		 
		This is the archive player client.
		 
		Functions for loading and parsing archive scene files.


******************************************************************************
efh*/


//
// Objects
//
var Debug = GetDebugObject();
var Params = GetParamsObject();
var D2E = GetDOM2EventObject();
var Engine = GetEngineObject();
var Comms = GetCommsObject();
var LocalAPI = GetLocalAPIObject();
var Zoom = GetZoomObject();
var ArchivePlayer = GetArchivePlayerObject();

function GetArchivePlayerObject()
{
	if (!GetArchivePlayerObject.ArchivePlayer)
	{
		GetArchivePlayerObject.ArchivePlayer = new ArchivePlayerObject;
	}
	return GetArchivePlayerObject.ArchivePlayer;
}

function InitPlayer() { ArchivePlayer.InitPlayer(); }

//
// BROWSER CLIENT API
// 
function GetFolder() { return ArchivePlayer.GetFolder(); }
function PauseAppshareArchive() { ArchivePlayer.PauseAppshareArchive(); }
function PlayAppshareArchive(playTimeMillis) { ArchivePlayer.PlayAppshareArchive(playTimeMillis); }
function IsPaused() { return ArchivePlayer.IsPaused(); }
function GetReadyState() { return ArchivePlayer.GetReadyState(); }
function SetSyncClock(externalTimeMillis) { ArchivePlayer.SetSyncClock(externalTimeMillis); }
function Seek(timeMillis) { ArchivePlayer.Seek(timeMillis); }
function GetFirstTimestamp() { return ArchivePlayer.GetFirstTimestamp(); }
function GetLastTimestamp() { return ArchivePlayer.GetLastTimestamp(); }
function GetCurrentTimestamp() { return ArchivePlayer.GetCurrentTimestamp(); }

//
// ArchivePlayerObject
//
function ArchivePlayerObject()
{
	//
	// Statics
	//
	//var s_CURSOR_PASSIVE_VIEWER_WITH_ZOOM = "url('magnify.cur'), crosshair";
	//var s_CURSOR_PASSIVE_VIEWER_NO_ZOOM = "url('buster.cur'), crosshair";

	//
	// Vars
	//
	var m_bSessionActive = false;
	var m_bActiveController = false;
	var m_xOffset = 0; 				// Screen offsets, remembered 
	var m_yOffset = 0;				// while Active Controller.
	
	var m_bSharingOn = 0;   //set to 1 if applications are being shared, 0 if nothing is shared
	
	// Archive Player
	var m_folder;
	
	var m_sceneCount;
	var m_crtSceneID;
	var m_sceneArray = new Array();
	var m_currentPosition = 0;
	
	var m_pngFilePath;			// full path to the current Scene png file
	var m_gifCursorsPath;		// full path to the Cursors gif file (cursors.gif)
	
	var m_bAppshareArchivePaused = false;	// true if the playback is paused
	var	m_bSeeking = false;					// true if seeking to new position
	var m_bMasterLoaded = false;			// true if the master scene list has been loaded
	var m_appshareEventTimer = 0;		    // timer to play a scene
	
	
	/*sfh
	******************************************************************************
	
		Function:		InitPlayer
	
		Desc:			Called when the page is first loaded.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.InitPlayer = function()
	{
		// this is the archive player
		Comms.SetArchivePlayerMode(true);
		
		// Grab the archive folder name. Folder is normally relative to current page.
		m_folder = Params.GetParam("folder");
		if (!m_folder)
		{
			Debug.ConsoleOut("No archive folder provided!");
			return;
		}
		
		// replace each , with '../'
		var re = new RegExp(",", "g");
		m_folder = m_folder.replace( re, "../");
		
		var lang = Params.GetParamWithDefault("lang", "us-en");
		Comms.FetchStrings(lang);
	
		// Check for IFRAME comms override - will force IFRAME simulator
		// instead of XMLHTTPRequest object for communications 
		var bForceIFRAME = Params.GetParamWithDefault("forceiframe", "0");
		if (Params.isTrue(bForceIFRAME))
		{
			Debug.ConsoleOut("Forcing IFRAME simulator for comms");
			Comms.SetForceIFRAMEComms(true);
		}
	
		var zoom = Params.GetParamWithDefault("zoom", "100");
		var bAutoscroll = Params.isTrue(Params.GetParamWithDefault("autoscroll", "0"));
	
		Debug.ConsoleOut("PlayerInit() -- folder = " + m_folder);
		Debug.ConsoleOut("PlayerInit() -- zoom = " + zoom);
		Debug.ConsoleOut("PlayerInit() -- autoscroll = " + bAutoscroll);	
		
		m_sceneCount = 0;
		m_gifCursorsPath = m_folder + "/cursors.gif";
		PreLoadArchive.sceneID = 0;

		Zoom.CreateZoomUI();
		LocalAPI.SetCallbackAPI(this);
	
		// there is no sessionID for appshplayer	
		var bZoomSupported = Engine.EngIsZoomSupported();
		Engine.EngEngineInit(zoom, bAutoscroll);
		//Engine.EngSetLocalCursor(bZoomSupported ? s_CURSOR_PASSIVE_VIEWER_WITH_ZOOM : s_CURSOR_PASSIVE_VIEWER_NO_ZOOM);

		// Set mouse event handlers
		document.onmousedown = null;  
		document.onmouseup = bZoomSupported ? Zoom.On_MouseUp : null;  
		document.onmousemove = bZoomSupported ? Zoom.On_MouseMove : null;  
		document.ondblclick = null;  
		document.onkeydown = null;  
		document.onkeyup = ArchivePlayer.HandleKeyboardEvent;  
		if (!Debug.IsDebug()) { document.oncontextmenu = ArchivePlayer.EatEvent; }
		document.onhelp = null;
		
		// the player starts by fetching and executing the master.js file
		this.FetchMasterJS();
	};

	/*sfh
	******************************************************************************
	
		Function:		EatEvent
	
		Desc:			Handles events that should simply be stopped from 
						propagating and suppresses the default behaviour.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.EatEvent = function(evt)
	{
		var event = D2E.CreateDOM2Event(evt, window.event);
		D2E.CancelBubbling(event);
	};

	/*sfh
	******************************************************************************
	
		Function:		GetFolder					     (PUBLIC API)
	
		Desc:			Returns the archive folder.
	
		Returns:		string
	
	******************************************************************************
	efh*/
	
	this.GetFolder = function()
	{
		return m_folder;
	};
	
	/*sfh
	******************************************************************************
	
		Function:		PauseAppshareArchive     (PUBLIC API)
	
		Desc:			Pauses the appsharing archive playback.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.PauseAppshareArchive = function()
	{
		// set playback as 'paused'
		m_bAppshareArchivePaused = true;
		
		// clear the play timer
		if( m_appshareEventTimer)
		{
			window.clearTimeout(m_appshareEventTimer);
			m_appshareEventTimer = 0;
		}	
		
		Debug.ConsoleOut("Appsharing archive playback PAUSED");
	};
	
	/*sfh
	******************************************************************************
	
		Function:		PlayAppshareArchive			(PUBLIC API)
	
		Desc:			Restarts the appsharing playback after Pause.
						The passed time is used to re-sync the internal playback clock.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.PlayAppshareArchive = function(playTimeMillis)
	{
		// adjust the clock
		this.SetSyncClock(playTimeMillis);
		
		// set playback as 'playing'
		m_bAppshareArchivePaused = false;
		ExecuteSceneCommands.lastClockSample = new Date();
		
		// restart the archive playback
		this.Seek(m_currentPosition);
	
		Debug.ConsoleOut("Appsharing archive playback UNPAUSED");
	};
	
	/*sfh
	******************************************************************************
	
		Function:		IsPaused     (PUBLIC API)
	
		Desc:			Returns true if archive playback is paused.
	
		Returns:		boolean
	
	******************************************************************************
	efh*/
	
	this.IsPaused = function()
	{
		return m_bAppshareArchivePaused;
	};
	
	/*sfh
	******************************************************************************
	
		Function:		GetReadyState					     (PUBLIC API)
	
		Desc:			Returns one of "loading", "loaded" or "seeking".
	
		Returns:		string
	
	******************************************************************************
	efh*/
	
	this.GetReadyState = function()
	{
		return m_bMasterLoaded ? (m_bSeeking ? "seeking" : "loaded") : "loading";
	};
	
	/**sfh
	******************************************************************************
	
		Function:		SetSyncClock				(PUBLIC API)
	
		Desc:			Sets the current clock time used for synchronization.
						When synchronizing with an external source, such as 
						an audio stream, this method should be called periodically
						with the current time in the audio stream.
						
						This method updates an offset which is relative to the
						machine clock, and therefore this method can be called
						once or many times depending on the amount of drift
						expected.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.SetSyncClock = function(externalTimeMillis)
	{
		m_currentPosition = externalTimeMillis;
	};
	
	/**sfh
	******************************************************************************
	
		Function:		Seek							(PUBLIC API)
	
		Desc:			Seeks (rewind or fastforward) to the specified timestamp.`
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.Seek = function(timeMillis)
	{
		var sceneID = FindScene(timeMillis);
	
		Debug.ConsoleOut("Seek called:  new scene=" + sceneID);
	
		if (sceneID != -1)
		{
			// CA - make sure the archive is not paused
			m_bAppshareArchivePaused = false;
			
			m_bSeeking = true;
			m_crtSceneID = sceneID;
			this.SetSyncClock(timeMillis);
			this.LoadScene(sceneID);
		}
		else
		{
			Debug.ConsoleOut("WARNING --- Seek out of range: " + timeMillis);
		}
	};
	
	/**sfh
	******************************************************************************
	
		Function:		GetFirstTimestamp				(PUBLIC API)
	
		Desc:			Returns the timestamp of the first event in the archive.
	
		Returns:		long
	
	******************************************************************************
	efh*/
	
	this.GetFirstTimestamp = function()
	{
		try { return m_sceneArray[0][2]; } catch (e) { return -1; }
	};
	
	/**sfh
	******************************************************************************
	
		Function:		GetLastTimestamp				(PUBLIC API)
	
		Desc:			Returns the timestamp of the last event in the archive.
	
		Returns:		long
	
	******************************************************************************
	efh*/
	
	this.GetLastTimestamp = function()
	{
		try { return m_sceneArray[m_sceneCount - 1][3]; } catch (e) { return -1; }
	};
	
	/**sfh
	******************************************************************************
	
		Function:		GetCurrentTimestamp				(PUBLIC API)
	
		Desc:			Returns the timestamp of the current event playing.
	
		Returns:		long
	
	******************************************************************************
	efh*/
	
	this.GetCurrentTimestamp = function()
	{
		return m_currentPosition;
	};
	
	/*sfh
	******************************************************************************
	
		Function:		FetchMasterJS
	
		Desc:			Fetches the master.js file using XMLHTTPRequest object.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.FetchMasterJS = function()
	{ 
	    // Load master.js
		var url = m_folder + "/master.js";
		Comms.CallAPI( this.FetchMasterJS, url, this.FetchMasterJS_APICallback);
	};
	
	/*sfh
	******************************************************************************
	
		Function:		FetchMasterJS_APICallback
	
		Desc:			Called when the download for master.js is completed.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.FetchMasterJS_APICallback = function(httpStatus)
	{ 
	    Debug.ConsoleOut("FetchMaster:  httpStatus=" + httpStatus);
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnSetDimensions
	
		Desc:			Called when screen & tile dimensions are provided.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnSetDimensions = function(numXTiles, numYTiles, tileWidth, tileHeight, screenWidth, screenHeight)
	{ 
		Debug.ConsoleOut("OnSetDimensions...");
		Debug.ConsoleOut("--- numTiles: " + numXTiles + " x " + numYTiles);
		Debug.ConsoleOut("------- tile: " + tileWidth + " x " + tileHeight);
		Debug.ConsoleOut("----- screen: " + screenWidth + " x " + screenHeight);	
		
		// Only do it once.
		if (!this.OnSetDimensions.done)
		{
			Engine.EngBuildTileMap(numXTiles, numYTiles, tileWidth, tileHeight, screenWidth, screenHeight);
			this.OnSetDimensions.done = true;
		}
	};
	
	/*sfh
	******************************************************************************
	
		Function:		LoadScene
	
		Desc:			Loads an archive Scene. First, download the the scene js 
						file using an HTTPRequest object.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.LoadScene = function(sceneID)
	{
		// First stop the command execution of any other scene already playing.
		if (m_appshareEventTimer) 
		{ 
			window.clearTimeout(m_appshareEventTimer); 
			m_appshareEventTimer = 0;
		}
	
		var sceneJS = m_sceneArray[sceneID][0];
		
		// Set the png file name
		m_pngFilePath = m_folder + "/" + m_sceneArray[sceneID][1];
		
		var req = Comms.CreateXMLHttpRequest(); 	
		if (!req)
		{	
			// XMLHTTPRequest object not supported.
			Debug.ConsoleOut("LoadScene ---  FAILED to create XMLHTTPRequest object");
			return; 
		} 
		
		var url = m_folder + "/" + sceneJS;
		Debug.ConsoleOut( "LoadScene --- ID=" + sceneID + " , JS Path=" + sceneJS + " , url=" + url);
	
		req.open("GET", url, true);
		req.onreadystatechange = function() { ArchivePlayer.OnLoadSceneReadyStateChange(req); };
		req.send("");
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnLoadSceneReadyStateChange
	
		Desc:			Callback when the scene js content is loaded.
						Start parsing the scene js file.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnLoadSceneReadyStateChange = function(req)
	{
		if (req && req.readyState == 4)
		{
			Debug.ConsoleOut("--- OnLoadSceneReadyStateChange - scene JS loaded");
		
			// Start playing the scene js file
			PlayScene(req.responseText);	
		}
	};
	
	/*sfh
	******************************************************************************
	
		Function:		PlayScene
	
		Desc:			Plays the scene. Each timestamp delimited section of
						JavaScript is parsed and executed using "eval".
						Section playback is delayed based on the timestamps in
						order to preserve the original timing.
	
						Calls to "SetSyncClock" can also be used to adjust the
						playback timing to synchronize with an external clock,
						such as streaming audio. 
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function PlayScene(strScene)
	{
		if (strScene || !PlayScene.section)
		{
			PlayScene.section = new Object();
			PlayScene.section.startTime = new Date();
			PlayScene.section.strCommands = "";
			PlayScene.section.strSceneRemaining = strScene;
		}
	
		PlayScene.section = ParseNextSection(PlayScene.section);
		if (PlayScene.section)
		{
			//ConsoleOut("PlayScene --- scene section parsed:");
			//ConsoleOutProps(PlayScene.section);
			ExecuteSceneCommands();
		}
	}
	
	/*sfh
	******************************************************************************
	
		Function:		ParseNextSection
	
		Desc:			Parses out the next timestamp and commands, populating a
						section object with this info. The field "strSceneRemaining"
						is updated with just the remaining portion of the scene. 
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function ParseNextSection(section)
	{
		var sectionOut = null;
		
		// Parse the next timestamp. 
		// Timestamps are in the format: // !![123456789]
		var iIdx = section.strSceneRemaining.indexOf("// !![");
		if( iIdx != -1)
		{
			var iIdx2 = section.strSceneRemaining.indexOf("]", iIdx);
			
			var startTime = parseInt(section.strSceneRemaining.substring(iIdx + 6, iIdx2 + 1));
			if (!isNaN(startTime))
			{
				section.startTime = new Date(startTime);
				
				section.strSceneRemaining = section.strSceneRemaining.substring(iIdx2 + 1, section.strSceneRemaining.length);
				
				// Now find the beginning of the next timestamp (or end of file).
				var iIdx3 = section.strSceneRemaining.indexOf("// !![");
				if (iIdx3 == -1) { iIdx3 = section.strSceneRemaining.length; }
				
				section.strCommands = section.strSceneRemaining.substring(0, iIdx3);
				section.strSceneRemaining = section.strSceneRemaining.substring(iIdx3, section.strSceneRemaining.length);
	
				sectionOut = section;			
			}
			else
			{
				Debug.ConsoleOut("GetNextSection --- ERROR parsing timestamp: " + section.strSceneRemaining.substring(iIdx+6,iIdx2));
			}
		}
		return sectionOut;
	}
	
	/*sfh
	******************************************************************************
	
		Function:		ExecuteSceneCommands
	
		Desc:			Executes a group of js commands from a scene.
						The commands are only executed when the clock reaches
						the timestamp associated with the scene section.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function ExecuteSceneCommands()
	{
		m_appshareEventTimer = 0;
		var rightNow = new Date();
	
		if (!ExecuteSceneCommands.lastClockSample)
		{
			ExecuteSceneCommands.lastClockSample = rightNow;
		}
	
		if (PlayScene.section && !m_bAppshareArchivePaused)
		{
			// Advance the current position based on how much time has elasped since last clock sample.
			m_currentPosition = m_currentPosition + (rightNow - ExecuteSceneCommands.lastClockSample);
			ExecuteSceneCommands.lastClockSample = rightNow;
	
			// Determine if it is now time to play the next event(s), otherwise sleep.
			var timeLeft = PlayScene.section.startTime - m_currentPosition;
	
			if (timeLeft <= 0 || m_bSeeking)
			{
				try
				{
					eval(PlayScene.section.strCommands);
				}
				catch(e)
				{
					Debug.ConsoleOut("ExecuteSceneCommands --- EXCEPTION executing commands: " + PlayScene.section.strCommands);
				}
		
				// Continue on to the next section.
				PlayScene();
			}
			else
			{
				//ConsoleOut("ExecuteSceneCommands --- not time yet ---- timeLeft= " + timeLeft);
				
				// Not time yet. Check again later.
				if (timeLeft > 1000) { timeLeft = timeLeft / 2; }
				
				m_appshareEventTimer = window.setTimeout("ArchivePlayer.ExecuteSceneCommands_Callback();", timeLeft);
			}
		}
	}

	/*sfh
	******************************************************************************
	
		Function:		ExecuteSceneCommands_Callback
	
		Desc:			Provides the loop for counting down until next command
						is to be executed.
	
		Returns:		void
	
	******************************************************************************
	efh*/

	this.ExecuteSceneCommands_Callback = function()
	{
		ExecuteSceneCommands();
	};
	
	/**sfh
	******************************************************************************
	
		Function:		FindScene
	
		Desc:			Finds the scene that contains the specified timestamp.
						Returns the scene id, or -1 if not found.
	
		Returns:		int (scene id or -1 if not found.)
	
	******************************************************************************
	efh*/
	
	function FindScene(timeMillis)
	{
		// Walk the scene array looking for the timestamp.
		var iScene = -1;
		var i = 0;
		
		while (m_sceneArray[i])
		{
			if (timeMillis >= m_sceneArray[i][2] && timeMillis <= m_sceneArray[i][3])
			{
				iScene = i;
				break;
			}
			i = i + 1;	
		}
		return iScene;
	}
	
	/*sfh
	******************************************************************************
	
		Function:		PreLoadArchive
	
		Desc:			Downloads the resources for all scenes in the archive.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function PreLoadArchive()
	{
		PreLoadArchive.sceneID = PreLoadArchive.sceneID + 1;
	
		// Check if we've gone past the last scene id 
		if (PreLoadArchive.sceneID < m_sceneCount)
		{
			PreLoadArchive.cachePNGDone = false;
			PreLoadArchive.cacheJSDone = false;
			
			var sceneJS = m_folder + "/" + m_sceneArray[PreLoadArchive.sceneID][0];
			var scenePNG = m_folder + "/" + m_sceneArray[PreLoadArchive.sceneID][1];
	
			//ConsoleOut("Starting preload of scene " + PreLoadArchive.sceneID);
			
			// Download the PNG
			PreLoadArchive.cachePNG = document.createElement("img");
			PreLoadArchive.cachePNG.onload = new Function("ArchivePlayer.On_IMG_PreloadScene();");
			PreLoadArchive.cachePNG.onerror = new Function("ArchivePlayer.On_IMG_PreloadScene();");
			PreLoadArchive.cachePNG.onabort = new Function("ArchivePlayer.On_IMG_PreloadScene();");
			PreLoadArchive.cachePNG.src = scenePNG;
	
			// Download the JS
			var req = Comms.CreateXMLHttpRequest(); 	
			if (req)
			{	
				req.open("GET", sceneJS, true);
				req.onreadystatechange = function() { ArchivePlayer.OnPreLoadSceneReadyStateChange(req); };
				req.send("");
			}
		}
		else
		{
			// We're done, clean up the cache.
			PreLoadArchive.cachePNG.src = null;
		}
	}
	
	/*sfh
	******************************************************************************
	
		Function:		On_IMG_PreloadScene
	
		Desc:			Called when a scene's PNG image is finished downloading 
						while preloading the archive.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.On_IMG_PreloadScene = function()
	{
		PreLoadArchive.cachePNGDone = true;
		
		// If both resources are finished then move on to next one.
		if (PreLoadArchive.cachePNGDone && PreLoadArchive.cacheJSDone)
		{
			// CR18161 - stack overflow in IE7 -> use timer instead of direct call
			window.setTimeout("ArchivePlayer.On_PreloadScene_Done();", 10);
		}
	};

	/*sfh
	******************************************************************************
	
		Function:		On_PreloadScene_Done
	
		Desc:			Called when a scene's PNG image and JS file is finished 
						downloading while preloading the archive.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.On_PreloadScene_Done = function()
	{
		// Move on the next scene.
		PreLoadArchive();
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnPreLoadSceneReadyStateChange
	
		Desc:			Called when a scene's JavaScript is finished downloading 
						while preloading the archive.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnPreLoadSceneReadyStateChange = function(req)
	{
		if (req && req.readyState == 4)
		{
			PreLoadArchive.cacheJSDone = true;
		}
	
		// If both resources are finished then move on to next one.
		if (PreLoadArchive.cachePNGDone && PreLoadArchive.cacheJSDone)
		{
			// CR18161 - stack overflow in IE7 -> use timer instead of direct call
			window.setTimeout("ArchivePlayer.On_PreloadScene_Done();", 10);
		}
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnAddScene
	
		Desc:			Called when master.js is eval-ed.
						Adds a scene to the Scenes array.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnAddScene = function(scenejs, scenejpg, starttime, endtime)
	{ 
	    //Debug.ConsoleOut("AddScene Called:  " + scenejs + " , " + scenejpg + " , " + starttime + " , " + endtime);
	    
	    // Use the start time of the first scene to set the clock offset, just in case
	    // there is no external clock.
	    if (m_sceneCount === 0)
	    {
	    	this.SetSyncClock(starttime); 
	    }
	    
	    var crtScene = new Array( scenejs, scenejpg, starttime, endtime);
		m_sceneArray[m_sceneCount] = crtScene;
		m_sceneCount = m_sceneCount + 1;
	};
	
	/**sfh
	******************************************************************************
	
		Function:		OnEndMaster
	
		Desc:			Called at the end of master.js processing.
						Starts playing the archive scenes, starting with sceneID 0.
						
						Also starts the preloading sequence, but only if using
						HTTP (versus FILE:)
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnEndMaster = function()
	{ 
		m_bMasterLoaded = true;
	
		// Start the preloading.
		var loc = document.location;
		if (loc.protocol == "http:" || loc.protocol == "https:")
		{
			PreLoadArchive();	
		}
	    
	    // start with the first scene
	    m_crtSceneID = 0;
	    m_currentPosition = this.GetFirstTimestamp();
	    
	    // play scene
	    window.setTimeout("ArchivePlayer.LoadScene(" + m_crtSceneID + ");", 1);
	
	    Debug.ConsoleOut("End Master Called, scene count=" + m_sceneCount + "  ; first timestamp=" + m_currentPosition);
	};
	
	/**sfh
	******************************************************************************
	
		Function:		OnEndScene
	
		Desc:			Called when a scene js file processing is completed.
						Plays the next scene, if any.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnEndScene = function()
	{
		//Debug.ConsoleOut("OnEndScene called");
		
		// play the next scene, if any
		m_crtSceneID = m_crtSceneID+1;
		if( m_crtSceneID < m_sceneCount)
		{
			window.setTimeout("ArchivePlayer.LoadScene(" + m_crtSceneID + ");", 1);
		}
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnSetTile
	
		Desc:			Called when a new archive tile is to be displayed.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnSetTile = function( tid, pngX, pngY)
	{
		Engine.EngSetTileCropped(tid, m_pngFilePath, pngX, pngY);
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnSetKeyFrameTile
	
		Desc:			Called to indicate the PNG image and coordinates of a key
						frame tile. The tile is to be displayed if we are seeking.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnSetKeyFrameTile = function(sid, tid, pngX, pngY)
	{
		if (m_bSeeking)
		{
			var pngFile = m_folder + "/" + m_sceneArray[sid][1];
			Engine.EngSetTileCropped(tid, pngFile, pngX, pngY);
			
			Debug.ConsoleOut("Setting keyframe tile: tid=" + tid + " ; x=" + pngX + " ; y="+ pngY + " ; png=" + pngFile);
		}
		else
		{
			//Debug.ConsoleOut("Ignoring keyframe tile: tid=" + tid);
		}
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnEndKeyFrame
	
		Desc:			Called to indicate that keyframe data for the scene is
						finished processing.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnEndKeyFrame = function()
	{
		m_bSeeking = false;
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnSetCursor
	
		Desc:			Called when the cursor image changes during archive playback.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnSetCursor = function( gifX, gifY, cursorWidth, cursorHeight)
	{
		Engine.EngSetCursorCropped(m_gifCursorsPath, gifX, gifY, cursorWidth, cursorHeight);
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnSetCursorPos
	
		Desc:			Called when new cursor coordinates have been provided by
						the server.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnSetCursorPos = function(x,y) 
	{  
		Engine.EngMoveCursor(x,y);
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnSetScreenOffset
	
		Desc:			Called when the top-left corner of the screen should be
						offset.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnSetScreenOffset = function(x,y) 
	{  
		m_xOffset = x;
		m_yOffset = y;
		
		if (!m_bActiveController)
		{
			Engine.EngSetScreenOffset(x,y);
		}
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnSetSharedRect
	
		Desc:			Called when the top-left corner of the screen should be
						offset, or the width/height of the shared display area
						changes.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnSetSharedRect = function(left, top, width, height) 
	{
		m_xOffset = left;
		m_yOffset = top;
		
		Engine.EngSetSharedRect(left, top, width, height);
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnShowTooltip
	
		Desc:			Called when the tooltip should be hidden or shown, or
						when the text changes. The tooltip follows the cursor
						and normally displays the name of the active viewer.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnShowTooltip = function(bShow, txt)
	{
		//Debug.ConsoleOut("OnShowTooltip: bShow=" + bShow + " --- txt=" + txt);
		Engine.EngShowTooltip(bShow, txt);
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnSetSharing
	
		Desc:			Called by the server when something is being shared, or
						when nothing is being shared. Aegis shows or hides the
						"No apps currently shared..." message.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnSetSharing = function(bSharing) 
	{
	    m_bSharingOn = bSharing;    // update the global variable
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnEndSession
	
		Desc:			Called by the server when the session is ended. No more
						requests for updates should be performed.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnEndSession = function()
	{
		m_bSessionActive = false;
		Debug.ConsoleOut("---Session Ended---");
	};

	/*sfh
	******************************************************************************
	
		Function:		HandleKeyboardEvent
	
		Desc:			Handles keyboard events captured by the document.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.HandleKeyboardEvent = function(evt)
	{
		var event = D2E.CreateDOM2Event(evt, window.event);

		// Check for the "force debug console"
		// key combination (ctrl+shift+tilde)
		if (event.type == "keyup" && event.ctrlKey && event.shiftKey && event.keyCode == 192)
		{
			Debug.SetForceDebug(true);
			Debug.ConsoleOut("Debug Console Forced On");
		}
	};

}// End of ArchivePlayerObject

// End of file