/**sfh
******************************************************************************

	appshareviewer
	~~~~~~~~~~~~~~
		 
		This is the live meeting appshare client.
		 
		Functions for starting appshare session, fetching updates, 
		and handling events from Athena server and local UI.


******************************************************************************
efh*/

//
// Objects
//
var Debug = GetDebugObject();
var Params = GetParamsObject();
var D2E = GetDOM2EventObject();
var Engine = GetEngineObject();
var Comms = GetCommsObject();
var LocalAPI = GetLocalAPIObject();
var Zoom = GetZoomObject();
var StreamMode = GetStreamModeObject();
var AppShareViewer = GetAppShareViewerObject();

function GetAppShareViewerObject()
{
	if (!GetAppShareViewerObject.AppShareViewer)
	{
		GetAppShareViewerObject.AppShareViewer = new AppShareViewerObject;
	}
	return GetAppShareViewerObject.AppShareViewer;
}

function InitClient() { AppShareViewer.InitClient(); }

//
// AppShareViewerObject
//

function AppShareViewerObject()
{
	//
	// Statics
	//
	var s_GET_UPDATE_WAITSTATE = 1;    // millis - wait time between calls to getupdate - keep very low
	var s_INPUT_EVENT_WAITSTATE = 150; // millis - wait time to recheck if any mouse/kb events to send if queue is empty
	var s_MAX_MOUSE_MOVES_IN_Q = 10;
	var s_MAX_EVENTS_IN_POST = 10;
	//var s_CURSOR_PASSIVE_VIEWER_WITH_ZOOM = "url('magnify.cur'), crosshair";
	//var s_CURSOR_PASSIVE_VIEWER_NO_ZOOM = "url('buster.cur'), crosshair";

	//
	// Vars
	//
	
	// State info
	var m_bSessionActive = false;
	var m_bActiveController = false;
	var m_lZoomBeforeActive = -1;	// remember zoom before Active, -1 is not initialized
	var m_xOffset = 0; 				// Screen offsets, remembered 
	var m_yOffset = 0;				// while Active Controller.
	var m_bDataAvailable = false;   // Indicates that the 'getnexttile' loop should start.
	
	// Strings
	var m_lang = "en";
	var m_nosharemsg = "Welcome to the Meeting Center<br /><br />Please wait while the moderator selects an application to share";
	var m_madeactivemsg = "Please click to gain control";
	var m_madepassivemsg = "The moderator has taken control";
	
	// Session
	var m_assid = null;							// Appshare server session id
	var m_cid = null;							// Appshare server client id
	var m_ck = null;							// Athena server content key
	var m_swck = null;  						// System-wide content key
	var m_displayname = null;   				// display name
	var m_confid = null;        				// ConferenceID

	/*sfh
	******************************************************************************
	
		Function:		InitClient
	
		Desc:			Calling when the page is first loaded.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.InitClient = function()
	{
		// this is the Aegis Live Client
		Comms.SetArchivePlayerMode(false);
		
		// Grab the appshare session id from the query string.
		m_assid = Params.GetParam("assid");
		if (!m_assid)
		{
			Debug.ConsoleOut("!! No assid provided!");
			return;
		}
	
		// Grab the system content key from the query string 
		// (used for login to Athena server).
		m_swck = Params.GetParam("ck");
		if (!m_swck)
		{
			Debug.ConsoleOut("!! No System Content Key provided!");
			return;
		}
	
		// Check for IFRAME comms override - will force IFRAME simulator
		// instead of XMLHTTPRequest object for communications 
		var bForceIFRAME = Params.GetParamWithDefault("forceiframe", "0");
		if (Params.isTrue(bForceIFRAME))
		{
			Debug.ConsoleOut("Forcing IFRAME simulator for comms");
			Comms.SetForceIFRAMEComms(true);
		}
	
		// Grab the participant's display name from the query string.
		m_displayname = escape(Params.GetParamWithDefault("dispname", ""));
		
		// Get the conference ID - used by Nike moderator to link the appsharing participent with the conference participant
		m_confid = Params.GetParamWithDefault("confid", "");
	
		// Language and strings.
		m_lang = Params.GetParamWithDefault("lang", m_lang);
		Comms.FetchStrings(m_lang);
	
		// Check for override to the default fetch mode.
		var fetchMode = Params.GetParamWithDefault("fetchmode", StreamMode.GetDefaultFetchMode());
		if (!StreamMode.IsValidFetchMode(fetchMode))
		{ 
			fetchMode = StreamMode.GetDefaultFetchMode(); // Revert to default.
			Debug.ConsoleOut("!! Invalid fetch mode in Init() url params: " + fetchMode);
		} 
		StreamMode.SetFetchMode(fetchMode);	
		
		// Get ZOOM and AutoScroll defaults
		var zoom = Params.GetParamWithDefault("zoom", "100");
		var bAutoscroll = Params.isTrue(Params.GetParamWithDefault("autoscroll", "0"));
	
		Debug.ConsoleOut("Init() -- assid = " + m_assid + " -- swck = " + m_swck);
		Debug.ConsoleOut("Init() -- zoom = " + zoom);
		Debug.ConsoleOut("Init() -- autoscroll = " + bAutoscroll);	
	
		Zoom.CreateZoomUI();
		LocalAPI.SetCallbackAPI(this);
	
		Engine.EngEngineInit(zoom, bAutoscroll);
		this.JoinSession(m_assid, m_swck, m_displayname, m_confid, StreamMode.GetFetchMode());
	};
	
	/*sfh
	******************************************************************************
	
		Function:		JoinSession
	
		Desc:			Calls the JoinSession API on the appshare server.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.JoinSession = function(assid, swck, displayname, confid, fetchmode)
	{
		var url;
		url = Comms.URL_JOIN_SESSION + "assid=" + assid + "/swck=" + swck + "/dispname=" + displayname + "/confid=" + confid;
		url = url + "/fetchmode=" + fetchmode + "/tilecachesize=" + StreamMode.GetDefaultTileCacheSize();
		url = url  + "/cursorcachesize=" + StreamMode.GetDefaultCursorCacheSize();
	
		Comms.CallAPI(this.JoinSession, url, this.JoinSession_APICallback);
	};
	
	/*sfh
	******************************************************************************
	
		Function:		JoinSession_APICallback
	
		Desc:			Called when the JoinSession API call is completed. HTTP
	                    status code may indicate success (200) or a communication
	                    error.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.JoinSession_APICallback = function(httpStatus)
	{
	    if (Debug.IsDebug())
	    {
	        Debug.ConsoleOut("JoinSession API Callback: http result = " + httpStatus);
	    }
	
	    // If communication error then retry.
	    if (httpStatus != 200)
	    {
			Debug.ConsoleOut("ERROR JoinSession: http result = " + httpStatus + "  --- retrying...");
	        window.setTimeout("AppShareViewer.InitClient();", 5000); // Wait 5 seconds  
		}
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnSetClientID
	
		Desc:			Called when a new client id is provided by the server.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnSetClientID = function(cid)
	{
		m_cid = cid;
		Engine.EngSetClientID(cid);
		Debug.ConsoleOut("Client ID Recieved: " + cid);
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnSetSessType
	
		Desc:			Called when the server sends the session type.
						(1-regular appsharing, 2-slideshow appsharing)
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnSetSessType = function(sesstype)
	{
		Engine.EngSetSessType(sesstype);
		Debug.ConsoleOut("Session Type Received: " + sesstype);
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnSetServerCk
	
		Desc:			Called when a new athena generated content key is 
						provided.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnSetServerCk = function(ck)
	{
		m_ck = ck;
		Debug.ConsoleOut("Content Key Received: " + ck);
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnSetDimensions
	
		Desc:			Called when new screen & tile dimensions are provided
						by the server.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnSetDimensions = function(numXTiles, numYTiles, tileWidth, tileHeight, screenWidth, screenHeight)
	{ 
		Debug.ConsoleOut("OnSetDimensions...");
		Debug.ConsoleOut("--- numTiles: " + numXTiles + " x " + numYTiles);
		Debug.ConsoleOut("------- tile: " + tileWidth + " x " + tileHeight);
		Debug.ConsoleOut("----- screen: " + screenWidth + " x " + screenHeight);	
	
		Engine.EngBuildTileMap(numXTiles, numYTiles, tileWidth, tileHeight, screenWidth, screenHeight);
	};

	/*sfh
	******************************************************************************
	
		Function:		OnSetFetchMode
	
		Desc:			Called when a new fetch mode is set by the server.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnSetFetchMode = function(fetchMode, tilecachesize, cursorcachesize)
	{
		if (StreamMode.IsValidFetchMode(fetchMode))
		{
			StreamMode.SetFetchMode(fetchMode);
			StreamMode.SetTileCacheSize(tilecachesize);
			StreamMode.SetCursorCacheSize(cursorcachesize);

			Debug.ConsoleOut("OnSetFetchMode - Fetch mode set to " + fetchMode); 
			Debug.ConsoleOut("OnSetFetchMode - tilecachesize=" + tilecachesize);
			Debug.ConsoleOut("OnSetFetchMode - cursorcachesize=" + cursorcachesize);
		}
		else
		{ 
			Debug.ConsoleOut("OnSetFetchMode - INVALID fetch mode requested by server: " + fetchMode);
		}
	};

	/*sfh
	******************************************************************************
	
		Function:		OnEndSession
	
		Desc:			Called by the server when the session is ended. No more
						requests for updates should be performed.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnEndSession = function()
	{
		m_bSessionActive = false;
		Debug.ConsoleOut("---Session Ended---");
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnCmdComplete_JoinSession
	
		Desc:			Called when a JoinSession API call has been completed
						successfully.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnCmdComplete_JoinSession = function(result)
	{
		m_bSessionActive = true;

		// Initialize the event handling for Active Controller and Zoom UI
		SetController(false, true); 
		
		// Now that we have screen layout and a client id, we can now
		// start polling for Updates.
		this.StartGetUpdatePolling();
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnCmdError_JoinSession
	
		Desc:			Called when a JoinSession API call returns an error
						from the server.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnCmdError_JoinSession = function(result, description)
	{
		Debug.ConsoleOut("OnCmdError_JoinSession:  result=" + result + "  ---  description=" + description);
	
		if (result == LocalAPI.SCODE_INVALID_CLIENTID)
		{
			// Invalid client id means the session is correct but it is not fully initialized
			// Try to re-join session.
			Debug.ConsoleOut("     --- Rejoining session " + m_assid + "...");
			this.JoinSession(m_assid, m_swck, m_displayname, m_confid);
		}
	};
	
	/*sfh
	******************************************************************************
	
		Function:		StartGetUpdatePolling
	
		Desc:			Starts continuously polling for updates from the server.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.StartGetUpdatePolling = function()
	{ 
	    // Create the GetUpdate URL.
		if (m_bSessionActive)
		{
			//Debug.ConsoleOut("StartGetUpdatePolling() - DataAvailable=" + m_bDataAvailable);
		
			if (m_bDataAvailable)
			{
				// s_FETCH_MODE_STREAM is active, and tiles are ready to be downloaded via 'getnexttile' server API.
				m_bDataAvailable = false;
				StreamMode.SetSessionInfo(m_assid, m_cid, m_ck);
				StreamMode.GetNextTile(this.StartGetUpdatePolling_APICallback);
			}
			else
			{
				// Call server API 'getupdate' for list of dirty tiles (s_FETCH_MODE_CACHE) 
				// or to check if any tiles ready (s_FETCH_MODE_STREAM).
				var url;
				url = Comms.URL_GET_UPDATE + "assid=" + m_assid + "/cid=" + m_cid;
				url = url + "/ck=" + m_ck;
			
				Comms.CallAPI(this.StartGetUpdatePolling, url, this.StartGetUpdatePolling_APICallback);	
			}
		}
	};
	
	/*sfh
	******************************************************************************
	
		Function:		StartGetUpdatePolling_APICallback
	
		Desc:			Called when a GetUpdate server API call is completed.
						Here we simply keep the loop going.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.StartGetUpdatePolling_APICallback = function(httpStatus)
	{ 
	    if ((typeof httpStatus != "undefined") && httpStatus != 200)
		{
			Debug.ConsoleOut("GetUpdate Callback ERROR:  http error " + httpStatus + " -- retrying");
		}
		
		// Wait a few milliseconds then start another GetUpdate.
	 	window.setTimeout("AppShareViewer.StartGetUpdatePolling();", s_GET_UPDATE_WAITSTATE);
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnCmdComplete_GetUpdate
	
		Desc:			Called when a GetUpdate API call has been completed
						successfully.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnCmdComplete_GetUpdate = function(result)
	{
		//Debug.ConsoleOut("GetUpdateComplete");
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnCmdError_GetUpdate
	
		Desc:			Called when a 'getupdate' API call returns an error
						from the server.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnCmdError_GetUpdate = function(result, description)
	{
		Debug.ConsoleOut("OnCmdError_GetUpdate:  result=" + result + "  ---  description=" + description);
	
		// If the error is for an invalid session id or invalid content key
		// then the error is unrecoverable and no more update requests should be
		// made.
		if (result == LocalAPI.SCODE_INVALID_SESSIONID || result == LocalAPI.SCODE_INVALID_CONTENTKEY)
		{
			Debug.ConsoleOut("     --- Ending Session");
			this.OnEndSession();	
		}
		else if (result == LocalAPI.SCODE_INVALID_CLIENTID)
		{
			// Invalid client id means the session is correct but this client has been
			// logged off for some reason. This can happen if a server fails-over, or
			// if the client connectivity was down for too long.
			// Try to re-join session.
			Debug.ConsoleOut("     --- Rejoining session " + m_assid + "...");
			this.JoinSession(m_assid, m_swck, m_displayname, m_confid);
		}
	};

	/*sfh
	******************************************************************************
	
		Function:		OnCmdError_GetNextTile
	
		Desc:			Called when a 'getnexttile' call returns an error
						from the server.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnCmdError_GetNextTile = function(result, description)
	{
		Debug.ConsoleOut("OnCmdError_GetNextTile:  result=" + result + "  ---  description=" + description);

		// Handle in same fashion as a 'getupdate' error.
		this.OnCmdError_GetUpdate(result, description);
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnDataAvailable
	
		Desc:			Called in response to a 'getupdate' request when in 
						fetch mode s_FETCH_MODE_STREAM and some data is available 
						to be fetched via 'getnexttile'.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnDataAvailable = function()
	{
		// Set the data available flag, which tells the GetUpdate loop to
		// stop until no more tiles are available, at which time the
		// GetUpdate loop is resumed.
		m_bDataAvailable = true;
	};	
	
	/*sfh
	******************************************************************************
	
		Function:		OnSetDirtyTiles
	
		Desc:			Called when a new set of dirty tile ids is provided by
						the server.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnSetDirtyTiles = function(count, format, args)
	{
		// Sanity check.
		if (count > 10000)
		{
			Debug.ConsoleOut("OnSetDirtyTiles:  more that 10K dirty tiles!!!");
		}
		else if (count > 0)
		{
			var arrComb;
			var str;
			var iTileID;
			var url;
			for (var i = 0;  i < count;  i = i + 1)
			{
				// Format: TileId_CaptureBatchId_CRC (CRC is optional)
				// If the tile id has two underscores then a CRC value is included.
				// Break the tile id apart from the CaptureBatchId and CRC and pass them separately.
				// Format is:  tid_crc_bid_bidl 
				str = new String(args[i+2]);
				arrComb = str.split("_", 4);
				
				iTileID = parseInt(arrComb[0]);
				if (!isNaN(iTileID))
				{
					var batchID = (arrComb[2]) ? parseInt(arrComb[2]) : 0;
					var crc = arrComb[1];
					var bLastInBatch = (arrComb[3]) ? parseInt(arrComb[3]) : 0;

					// If the tile id is negative then we are to use the background tile image (by sending null url)
					// otherwise generate a url via the Comms object.
					url = (iTileID < 0) ? null : Comms.GetTileURL(iTileID, crc, format, m_assid, m_ck);

					Debug.ConsoleOut("OnSetDirtyTiles -- tid,bid,url = " + iTileID + "," + batchID + "," + url);

					// Force engine to download a new image for this tile.					
					Engine.EngMarkTileDirty(iTileID, batchID, url, bLastInBatch);
				}
				else
				{
					Debug.ConsoleOut("OnSetDirtyTiles -- tile id is NaN: " + arrComb[0]);
				}
			}
		}
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnSetCursorImageDirty
	
		Desc:			Called when a new cursor image url has been provided by
					    the server.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnSetCursorImageDirty = function(hexCRC, format)
	{
		var url = Comms.GetCursorURL(hexCRC, format, m_assid, m_ck);
		Engine.EngMarkCursorImageDirty(url);
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnSetCursorPos
	
		Desc:			Called when new cursor coordinates have been provided by
						the server.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnSetCursorPos = function(x,y) 
	{  
		Engine.EngMoveCursor(x,y);
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnShowTooltip
	
		Desc:			Called when the tooltip should be hidden or shown, or
						when the text changes. The tooltip follows the cursor
						and normally displays the name of the active viewer.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnShowTooltip = function(bShow, txt)
	{
		Debug.ConsoleOut("OnShowTooltip: bShow=" + bShow + " --- txt=" + txt);
		Engine.EngShowTooltip(bShow, txt);
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnSetScreenOffset
	
		Desc:			Called when the top-left corner of the screen should be
						offset.
	
						DEPRECATED
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnSetScreenOffset = function(x,y) 
	{  
		m_xOffset = x;
		m_yOffset = y;
		
		// CA - execute it even when Aegis is an active viewer
		Engine.EngSetScreenOffset(x,y);
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnSetSharedRect
	
		Desc:			Called when the top-left corner of the screen should be
						offset, or the width/height of the shared display area
						changes.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnSetSharedRect = function(left, top, width, height) 
	{  
		m_xOffset = left;
		m_yOffset = top;
		
		// CA - execute it even when Aegis is an active viewer
		Engine.EngSetSharedRect(left, top, width, height);
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnSetSharing
	
		Desc:			Called by the server when something is being shared, or
						when nothing is being shared. Aegis shows or hides the
						"No apps currently shared..." message.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnSetSharing = function(bSharing) 
	{
	    Engine.EngNotifySharing(bSharing);
	    
		Debug.ConsoleOut("OnSetSharing:  bSharing=" + bSharing);
		var strNoShare = Comms.FetchStrings.stringtable[Comms.ATH_STATUS_NO_SHARE] ? 
							Comms.FetchStrings.stringtable[Comms.ATH_STATUS_NO_SHARE] : m_nosharemsg;
		this.ShowMessageBox(!bSharing, strNoShare);
	};

	/*sfh
	******************************************************************************
	
		Function:		OnSetController
	
		Desc:			Called by the server when this Aegis client has been 
						granted control or had it revoked.
						Being granted control means any mouse or keyboard events
						should be sent to the server, which are then pumped into
						the moderator's shared application.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnSetController = function(bController) 
	{  
		SetController(bController, false);
	};
	
	/*sfh
	******************************************************************************
	
		Function:		SetController
	
		Desc:			Sets up event handlers for ActiveController mode or
						for zoom UI.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function SetController(bController, bSuppressUI) 
	{  
		Debug.ConsoleOut("OnSetController:  bController=" + bController + " --- msgbox=" + (SetController.msgbox ? 'true':'false'));
		var bZoomSupported = Engine.EngIsZoomSupported();
		
		//Engine.EngSetLocalCursor(bController ? "default" : (bZoomSupported ? s_CURSOR_PASSIVE_VIEWER_WITH_ZOOM : s_CURSOR_PASSIVE_VIEWER_NO_ZOOM));
		
		if (bController && !SetController.msgbox)
		{
			var strActive = Comms.FetchStrings.stringtable[Comms.ATH_STATUS_MADE_ACTIVE] ? 
							Comms.FetchStrings.stringtable[Comms.ATH_STATUS_MADE_ACTIVE] : m_madeactivemsg;
			AppShareViewer.ShowMessageBox(true, strActive, 0);
			SetController.msgbox = true;
			document.onmousedown = AppShareViewer.OnClickToGainControl;
		}
		else if (bController)
		{
			AppShareViewer.ShowMessageBox(false);
			SetController.msgbox = false;
			
			m_bActiveController = true;

			// CR20648 - don't reset/restore the offset for active viewer
			//Engine.EngSetScreenOffset(0,0);
			
			// CA - Oct 11 - CR21030
			Zoom.ShowZoomUI(false);
			
			// CA - Oct 11 - CR21001 - reset zoom to 100% while viewer is active
			m_lZoomBeforeActive = Engine.EngGetZoom();
			Engine.EngSetZoom(100);
	
			// Add the event handlers for mouse and keyboard.
			document.onmousedown = AppShareViewer.HandleMouseEvent;  
			document.onmouseup = AppShareViewer.HandleMouseEvent;  
			document.onmousemove = AppShareViewer.HandleMouseEvent;  
			document.ondblclick = AppShareViewer.HandleMouseEvent;  
			document.onkeydown = AppShareViewer.HandleKeyboardEvent;  
			document.onkeyup = AppShareViewer.HandleKeyboardEvent;  
			document.oncontextmenu = AppShareViewer.EatEvent;
			document.onhelp = AppShareViewer.EatEvent;
			
			// Start up the event queue processor.
			ProcessInputEventQueue.bActive = true; 
			window.setTimeout("AppShareViewer.OnProcessInputEventQueue();", s_INPUT_EVENT_WAITSTATE);
		}
		else
		{
			// Stop message queue processing.
			ProcessInputEventQueue.bActive = false; 
			m_bActiveController = false;
			SetController.msgbox = false;
	
			// Remove the event handlers for mouse and keyboard.
			document.onmousedown = null;  
			document.onmouseup = bZoomSupported ? Zoom.On_MouseUp : null;  
			document.onmousemove = bZoomSupported ? Zoom.On_MouseMove: null;  
			document.ondblclick = null;
			document.onkeydown = AppShareViewer.HandleKeyboardEvent;  
			document.onkeyup = AppShareViewer.HandleKeyboardEvent;  
			document.oncontextmenu = AppShareViewer.EatEvent;
			document.onhelp = null;
		
			// Empty the Input Event queues.
			ProcessInputEventQueue.queueMajor = new Array();
			ProcessInputEventQueue.queueMinor = new Array();
		
			// CR20648 - don't reset/restore the offset for active viewer
			// Restore the screen offset.
			//Engine.EngSetScreenOffset(m_xOffset, m_yOffset);
			
			// CA - Oct 11 - CR21001 - restore zoom after viewer is demoted
			if(m_lZoomBeforeActive != -1)
			{
				Engine.EngSetZoom(m_lZoomBeforeActive);
				m_lZoomBeforeActive = -1;
			}
	
			if (!bSuppressUI)
			{
				var strPassive = Comms.FetchStrings.stringtable[Comms.ATH_STATUS_MADE_PASSIVE] ? 
								 Comms.FetchStrings.stringtable[Comms.ATH_STATUS_MADE_PASSIVE] : m_madepassivemsg;
				AppShareViewer.ShowMessageBox(true, strPassive, 5000);
			}
		}
	}
	
	/*sfh
	******************************************************************************
	
		Function:		OnClickToGainControl
	
		Desc:			Called when the "Click to gain control" message box is
						displayed and the user clicks the mouse.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnClickToGainControl = function(evt)
	{
		SetController(true, false);
	};
	
	/*sfh
	******************************************************************************
	
		Function:		HandleMouseEvent
	
		Desc:			Handles mouse event captured by the document.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.HandleMouseEvent = function(evt)
	{
		// Add mouse event to outbound queue. There are two queues, 
		// one for messages that must be delivered, and the other
		// for messages that can be dropped (mouse moves).
		var event = D2E.CreateDOM2Event(evt, window.event);
		var bMouseMove = (event.type == "mousemove") ? true : false;
		
		D2E.CancelBubbling(event);
		
		var obj = new Object();
		obj.mouse = true;
		obj.evtid = D2E.GetWindowsMouseEventID(event);
		
		// CA - adjust pos with top-left offset and zoom factor
		var iCrtZoom = Engine.EngGetActualZoom();
		var zoomPercent = iCrtZoom / 100;
		
		var xPos = event.pageX / zoomPercent + m_xOffset;
		var yPos = event.pageY / zoomPercent + m_yOffset;
		
		obj.x = xPos;
		obj.y = yPos;
		obj.wparam = (bMouseMove) ? D2E.GetWindowsKeyStateMask(event) : 0;
		
		// Push onto the end of the queue.
		if (bMouseMove)
		{
			// Don't let too many mouse moves build up. Kill the oldest.
			if (ProcessInputEventQueue.queueMinor.length > s_MAX_MOUSE_MOVES_IN_Q)
			{
				ProcessInputEventQueue.queueMinor.shift();
			}
			ProcessInputEventQueue.queueMinor.push(obj);
		}
		else
		{
			ProcessInputEventQueue.queueMajor.push(obj);
			// Dump any messages in the minor queue.
			if (ProcessInputEventQueue.queueMinor.length)
			{
				ProcessInputEventQueue.queueMinor = new Array();	
			}
		}
	};
	
	/*sfh
	******************************************************************************
	
		Function:		HandleKeyboardEvent
	
		Desc:			Handles keyboard events captured by the document.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.HandleKeyboardEvent = function(evt)
	{
		var event = D2E.CreateDOM2Event(evt, window.event);

		if (m_bActiveController)
		{
			D2E.CancelBubbling(event);

			var obj = new Object();
			obj.mouse = false;
			obj.evtid = 515;  // INPUT_KEY_EVENT. TODO: INPUT_SYSKEY_EVENT?
			obj.wparam = event.keyCode;
			obj.lparam = (event.type == "keyup") ? 0x80000000 : 0; // KEY_UP_LPARAM
			obj.cl = 0;  // TODO: CapsLock
			obj.sl = 0;  // TODO: ScrollLock
	
			// Add keyboard event to outbound queue.
			ProcessInputEventQueue.queueMajor.push(obj);
		}
		else
		{
			// Not active controller, so just check for the "force debug console"
			// key combination (ctrl+shift+tilde)
			if (event.type == "keyup" && event.ctrlKey && event.shiftKey && event.keyCode == 192)
			{
				Debug.SetForceDebug(true);
				Debug.ConsoleOut("Debug Console Forced On");
			}
		}
	};
	
	/*sfh
	******************************************************************************
	
		Function:		EatEvent
	
		Desc:			Handles events that should simply be stopped from 
						propagating and suppresses the default behaviour.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.EatEvent = function(evt)
	{
		var event = D2E.CreateDOM2Event(evt, window.event);
		D2E.CancelBubbling(event);
	};

	/*sfh
	******************************************************************************
	
		Function:		OnProcessInputEventQueue
	
		Desc:			Called when the event queue should be processed.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnProcessInputEventQueue = function()
	{
		ProcessInputEventQueue();
	};
	
	/*sfh
	******************************************************************************
	
		Function:		ProcessInputEventQueue
	
		Desc:			Processes the InputEventQueue by taking next message from
						the queue and sending to the Athena server. InputEvents
						are mouse and keyboard events generated locally that are 
						queued before sending to Athena.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function ProcessInputEventQueue()
	{
		// Grab next message from the queue and send to server.
		// Always grab from queueMajor first.
		var queue = (ProcessInputEventQueue.queueMajor.length) ? ProcessInputEventQueue.queueMajor : 
							  									 ProcessInputEventQueue.queueMinor;
		if (queue.length)
		{
			// Check if a request is already in progress.
			if (!ProcessInputEventQueue.req || ProcessInputEventQueue.req.readyState === 0 || ProcessInputEventQueue.req.readyState == 4)
			{
				var url;
				url = Comms.URL_UPDATE_CURSORKB2 + "assid=" + m_assid + "/cid=" + m_cid + "/ck=" + m_ck;
				
				var data = "";
				var nEvtCount = 0;
				var obj;
				
				while (nEvtCount < s_MAX_EVENTS_IN_POST)
				{
					obj = queue.shift();
					if (obj)
					{
						if (obj.mouse)
						{
							data = data + "updatecursor(" + obj.evtid + "," + obj.x + "," + obj.y + "," + obj.wparam + ");";
						}
						else
						{
							data = data + "updatekb(" + obj.evtid + "," + obj.wparam + "," + obj.lparam + "," + obj.cl + "," + obj.sl + ");";
						}
						
						nEvtCount = nEvtCount + 1;
					}
					else
					{
						break;
					}
				}
				
				if( data !== "")
				{
					data = data + "endupdate();";
				
					//if (Debug.IsDebug()) { Debug.ConsoleOut("CallPostAPI:  url=" + url + "data=" + data); }
					Comms.CallAPI(ProcessInputEventQueue, url, null, data);
				}
			}
		}
		
		// Continue processing.
		if (ProcessInputEventQueue.bActive)
		{
			// If there are more messages in queue then process them immmediately, otherwise wait a while
			// before checking the queue again.
			var iWaitState = (queue.length) ? 1 : s_INPUT_EVENT_WAITSTATE;
			window.setTimeout("AppShareViewer.OnProcessInputEventQueue();", iWaitState);	
		}
	}
	
	ProcessInputEventQueue.bActive = false;
	ProcessInputEventQueue.queueMajor = new Array();
	ProcessInputEventQueue.queueMinor = new Array();

	/*sfh
	******************************************************************************
	
		Function:		OnCmdComplete_UpdateCursor
	
		Desc:			Called when the server API call to update the cursor 
						position is complete. Updating the cursor is only done
						when this client is in "Active Controller" mode, meaning
						all keyboard and mouse events are sent to the server.
	
		Returns:		void
	
	******************************************************************************
	efh*/

	this.OnCmdComplete_UpdateCursor = function(result)
	{
		if (Debug.isDebug()) { Debug.ConsoleOut("OnCmdComplete_UpdateCursor: result=" + result); }
	};

	/*sfh
	******************************************************************************
	
		Function:		OnCmdComplete_UpdateKeystroke
	
		Desc:			Called when the server API call to post a keystroke is
						complete. Updating the cursor is only done when this 
						client is in "Active Controller" mode, meaning
						all keyboard and mouse events are sent to the server.
	
		Returns:		void
	
	******************************************************************************
	efh*/

	this.OnCmdComplete_UpdateKeystroke = function(result)
	{
		if (Debug.isDebug()) { Debug.ConsoleOut("OnCmdComplete_UpdateKeystroke: result=" + result); }
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnCmdComplete_UpdateCursorKb2
	
		Desc:			Called when the server API call to post a mouse or keystroke 
						event is complete.
	
		Returns:		void
	
	******************************************************************************
	efh*/

	this.OnCmdComplete_UpdateCursorKb2 = function(result)
	{
		if (Debug.isDebug()) { Debug.ConsoleOut("OnCmdComplete_UpdateCursorKb2: result=" + result); }
	};
	
	/*sfh
	******************************************************************************
	
		Function:		ShowMessageBox
	
		Desc:			Shows or hides the messagebox DIV and writes the msg 
						to the DIV's contents.
						
						Timeout determines how long the message box should be 
						displayed before automatically disappearing.
						Timeout = 0 means display indefinitely.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.ShowMessageBox = function(bShow, msg, timeout)
	{
		if (this.ShowMessageBox.timeout)
		{
			window.clearTimeout(this.ShowMessageBox.timeout);
			this.ShowMessageBox.timeout = 0;
		}
	
		var div = document.getElementById("divMessageBox");
		var txt = document.getElementById("pMessageBoxText");
		var imgMessageBox = document.getElementById("imgMessageBox");
		
		txt.innerHTML = (msg) ? msg : "";
		div.style.visibility = (bShow) ? "visible" : "hidden";
		imgMessageBox.style.visibility = (bShow) ? "visible" : "hidden";
		
		if (timeout)
		{
			this.ShowMessageBox.timeout = window.setTimeout("AppShareViewer.ShowMessageBox(false);", timeout);
		}
	};

} // End of AppShareViewerObject

// End of file