/*sfh
******************************************************************************

 Communications Library
 
******************************************************************************
efh*/

var Debug = GetDebugObject();
var D2E = GetDOM2EventObject();
var Comms = GetCommsObject();

function GetCommsObject()
{
	if (!GetCommsObject.comms)
	{
		GetCommsObject.comms = new CommsLibObject();
	}
	return GetCommsObject.comms;
}

//
// CommsLibObject
//

function CommsLibObject()
{
	//
	// Statics
	//
	this.URL_BASE = "/Athena/TileGen/"; 
	this.URL_JOIN_SESSION = this.URL_BASE + "cmd=joinsession/";
	this.URL_GET_UPDATE = this.URL_BASE + "cmd=getupdate/";
	this.URL_GET_TILE = this.URL_BASE + "cmd=gettile/";
	this.URL_GET_NEXT_TILE = this.URL_BASE + "cmd=getnexttile/";
	this.URL_GET_CURSOR = this.URL_BASE + "cmd=getcursor/";
	this.URL_UPDATE_CURSOR = this.URL_BASE + "cmd=updatecursor/";
	this.URL_UPDATE_KB = this.URL_BASE + "cmd=updatekb/";
	this.URL_UPDATE_CURSORKB2 = this.URL_BASE + "cmd=updatecursorkb2/";
	
	this.URL_CREATE_SESSION = this.URL_BASE + "cmd=createsession/";
	this.URL_SET_IMAGE = this.URL_BASE + "cmd=setimage/";
	this.URL_SET_CURSOR = this.URL_BASE + "cmd=setcursor/";
	
	this.URL_FETCH_STRINGS = "";
	this.ATH_STATUS_MADE_ACTIVE = "ATH_STATUS_MADE_ACTIVE="; 
	this.ATH_STATUS_MADE_PASSIVE = "ATH_STATUS_MADE_PASSIVE=";
	this.ATH_STATUS_NO_SHARE = "ATH_STATUS_NO_SHARE="; 
	this.ATH_ZOOM_AUTO = "ATH_ZOOM_AUTO=";
	this.ATH_ZOOM_WIDTH = "ATH_ZOOM_WIDTH=";
	this.ATH_ZOOM_TOOLTIP = "ATH_ZOOM_TOOLTIP=";
	this.STRINGS_TO_FETCH = [this.ATH_STATUS_MADE_ACTIVE, this.ATH_STATUS_MADE_PASSIVE, this.ATH_STATUS_NO_SHARE, this.ATH_ZOOM_AUTO, this.ATH_ZOOM_WIDTH, this.ATH_ZOOM_TOOLTIP];
	
	this.REQUEST_TIMEOUT = 90000; 		// ms
	
	//
	// Vars
	//
	var that = this;
	
	var bForce_IFrame_Request = false;  // If true, forces an IFRAME to be used 
									    //for comms instead of XMLHTTPRequest object
	
	this.bArchivePlayer = false;		// True if this is in archive playback mode
	
	
	/*sfh
	******************************************************************************
	
		Function:		GetTileURL
	
		Desc:			Creates a URL for the specified tile id.
	
		Returns:		string
	
	******************************************************************************
	efh*/
	
	this.GetTileURL = function(tileID, hexCRC, format, assid, ck)
	{
		var url = that.URL_GET_TILE + "assid=" + assid + "/ck=" + ck + "/tid=" + tileID;
	
		// If a CRC is provided then use that to make the tile URL unique so that the
		// same tile image can be cached for multiple requests.
		if (!hexCRC)
		{
			url = url + "/time=" + new Date().getTime();
		}
		else
		{
			url = url + "/crc=" + hexCRC;
		}
		url = url + "/ext=." + format;
		return url; 
	};
	
	/*sfh
	******************************************************************************
	
		Function:		GetCursorURL
	
		Desc:			Creates a URL for the appsharing mouse cursor.
	
		Returns:		string
	
	******************************************************************************
	efh*/
	
	this.GetCursorURL = function(hexCRC, format, assid, ck)
	{
		var url = that.URL_GET_CURSOR + "assid=" + assid + "/ck=" + ck;
	
		// If a CRC is provided then use that to make the URL unique so that the
		// same cursor image can be cached for multiple requests.
		if (!hexCRC)
		{
			url = url + "/time=" + new Date().getTime();
		}
		else
		{
			url = url + "/crc=" + hexCRC;
		}
		url = url + "/ext=." + format;
		return url; 
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnAPIReadyStateChange
	
		Desc:			Called when one of the XMLHttpRequest objects has a state
						change when performing an Athena API call. 
						
						If the request is complete, then the reponse is executed
						as JavaScript using eval().
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnAPIReadyStateChange = function(req, notify)
	{
		if (req && req.readyState == 4)
		{
	        // On network error, FireFox 1.5 throws an exception
	        // when checking req.status, where IE and Opera return 200.
	        var httpStatus = 0;
	        try
	        {
	            httpStatus = req.status;
	        }
	        catch(ee)
	        {
	            httpStatus = 0;
	        }
	        
	        try
	        {
	            if (notify)
	            {
	                notify(httpStatus);
	            }
	        }
	        catch(e1)
	        {
	            Debug.ConsoleOut("Notify ERROR - " + e1.description);
	        }
	        
	        try
			{
	            if (httpStatus == 200 || httpStatus == 304 || (httpStatus === 0 && req.responseText))
	            {
	                var txtClean = SanitizeJS(req.responseText);
	                
	                //Debug.ConsoleOut("gonna eval (status=" + httpStatus + "): " + txtClean);
	                
	                eval(txtClean);
	            }
	            else
	            {
	                throw new Error(req.statusText);
	            }
			}
			catch(e2) 
			{
	            Debug.ConsoleOut("-------------------------------");
	            Debug.ConsoleOut("---- ERROR on API Call: " + e2.description);
	            Debug.ConsoleOut("-------------------------------");
	            Debug.ConsoleOut("---- status          : " + (httpStatus!==0) ? req.status : "");			
	            Debug.ConsoleOut("---- statusText      : " + (httpStatus!==0) ? req.statusText : "");			
	            Debug.ConsoleOut("---- responseText    : " + (httpStatus!==0) ? req.responseText : "");
	            Debug.ConsoleOut("---- responseXML     : " + (httpStatus!==0) ? req.responseXML : "");
	            Debug.ConsoleOut("---- responseHeaders : " + (httpStatus!==0) ? req.getAllResponseHeaders() : "");
	            Debug.ConsoleOut("---- fileName        : " + e2.fileName);
	            Debug.ConsoleOut("---- lineNumber      : " + e2.lineNumber);
	            Debug.ConsoleOut("---- message         : " + e2.message);
	            Debug.ConsoleOut("---- name            : " + e2.name);
	            Debug.ConsoleOut("---- number          : " + e2.number);			
	            Debug.ConsoleOut("---- stack           : " + e2.stack);			
	            Debug.ConsoleOut("-------------------------------");
			}
		}
	};
	
	/*sfh
	******************************************************************************
	
		Function:		SanitizeJS
	
		Desc:			Removes leading "throw" or comment tags used to protect
					    the script from "JavaScript Hack" attack.
	
		Returns:		string
	
	******************************************************************************
	efh*/
	
	function SanitizeJS(txtScript)
	{
		txtScript = Trim(txtScript);
	
		var table = [["/*","*/"], ["throw me;",""], ["",""]];
		var str;
	
		for (var i = 0;  i < table.length;  i=i+1)
		{
			// Check for prefix
			str = table[i][0];
			if (str.length > 0 && str.length <= txtScript.length)
			{
				if (txtScript.substring(0, str.length) == str)
				{
					txtScript = txtScript.substring(str.length);
				}
			}
			// Check for suffix
			str = table[i][1];
			if (str.length > 0 && str.length <= txtScript.length)
			{
				if (txtScript.substring(txtScript.length - str.length) == str)
				{
					txtScript = txtScript.substring(0, txtScript.length - str.length);
				}
			}
		}
		return txtScript;
	}
	
	/*sfh
	******************************************************************************
	
		Function:		Trim
	
		Desc:			Removes whitespace from beginning and end of string. 
	
		Returns:		string
	
	******************************************************************************
	efh*/
	
	function Trim(stringToTrim)
	{
		if (stringToTrim === null || stringToTrim.length === 0) { return stringToTrim; }
	
		var strWhitespace = " \t\n\r";
		var iPos = 0;
		var theChar;
	
		// Find the first non-whitespace character.
		while (iPos < stringToTrim.length)
		{
			theChar = stringToTrim.charAt(iPos);
			if (strWhitespace.indexOf(theChar) == -1) {	break; }
			iPos = iPos + 1;
		}
		stringToTrim = stringToTrim.substring(iPos);
		
		// Find the last non-whitespace character.
		iPos = stringToTrim.length - 1; 
		while (iPos >= 0)
		{
			theChar = stringToTrim.charAt(iPos);
			if (strWhitespace.indexOf(theChar) == -1) {	break; }
			iPos = iPos - 1;
		}
		stringToTrim = stringToTrim.substring(0, iPos + 1);
		
		return stringToTrim;
	}
	
	/*sfh
	******************************************************************************
	
		Function:		CallAPI
	
		Desc:			Calls the server API specified by 'url'.
						If 'data' is not empty, calls the server API specified by 'url' 
						as http POST sending the 'data' string as the POST payload
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.CallAPI = function(caller, url, notify, data)
	{
		if (!caller.req)
		{
			caller.req = this.CreateXMLHttpRequest(); 	
		}
		
		if (!caller.req)
		{	
			// XMLHTTPRequest object not supported. 
			Debug.ConsoleOut("Failed to create XMLHTTPRequest object");
			return; 
		} 
			
		if (caller.req.readyState !== 0 && caller.req.readyState != 4)
		{	
			caller.req.abort();  
		}
		
		if( !this.bArchivePlayer)
		{
			url = url + "/time=" + new Date().getTime();
		}
				
		if( typeof data == "undefined")
		{
			//Debug.ConsoleOut("CallAPI [GET]: " + url);
		
			caller.req.open("GET", url, true);
			caller.req.onreadystatechange = function() { Comms.OnAPIReadyStateChange(caller.req, notify); };
			caller.req.send("");
		}
		else
		{
			Debug.ConsoleOut("CallAPI [POST]: " + url + " data = " + data);
		
			caller.req.open("POST", url, true);
			caller.req.onreadystatechange = function() { Comms.OnAPIReadyStateChange(caller.req, notify); };
			caller.req.send(data);
		}
	};
	
	/*sfh
	******************************************************************************
	
		Function:		SetArchivePlayerMode
	
		Desc:			Puts this comms library into archive player mode.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.SetArchivePlayerMode = function(bPlayer)
	{
		that.bArchivePlayer = bPlayer ? true : false;
	};
	
	/*sfh
	******************************************************************************
	
		Function:		GetArchivePlayerMode
	
		Desc:			Returns true if this comms library is in archive player 
						mode.
	
		Returns:		boolean
	
	******************************************************************************
	efh*/
	
	this.GetArchivePlayerMode = function()
	{
		return that.bArchivePlayer;
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OnFetchStringsReadyStateChange
	
		Desc:			Called when the ready state changes for the fetch strings
						request.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnFetchStringsReadyStateChange = function(req)
	{
		if (req && req.readyState == 4)
		{
			Debug.ConsoleOut("String fetch complete: " + req.status + " - " + req.statusText);
			
			// Try to parse out the strings we need and store them in the string 
			// table.
			var src = req.responseText;
			var iBegin, iEnd;
			
			if (src)
			{
				for(var i=0;  i < that.STRINGS_TO_FETCH.length;  i = i + 1)
				{
					iBegin = src.indexOf(that.STRINGS_TO_FETCH[i]);
					if (iBegin != -1)
					{
						iBegin = iBegin + that.STRINGS_TO_FETCH[i].length;
						iEnd = src.indexOf('\n', iBegin);
						if (iEnd != -1)
						{
							that.FetchStrings.stringtable[that.STRINGS_TO_FETCH[i]] = src.substring(iBegin, iEnd);
						}				
					}
				}
				Debug.ConsoleOutProps(that.FetchStrings.stringtable);
				
				// Remember that the string table has been fetched.
				that.FetchStrings.bFetchComplete = true;
			}
		}
	};
	
	/*sfh
	******************************************************************************
	
		Function:		FetchStrings
	
		Desc:			Downloads the string resource file for the specified
						language. The strings are stored in the stringtable,
						which is a property of this method.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.FetchStrings = function(lang)
	{
		that.FetchStrings.bFetchComplete = false;

		// Create the stringtable.
		if (!that.FetchStrings.stringtable)
		{
			that.FetchStrings.stringtable = new Array();
		}
	
		var req = this.CreateXMLHttpRequest(); 	
		if (!req)
		{	
			// XMLHTTPRequest object not supported. 
			Debug.ConsoleOut("Failed to create XMLHTTPRequest object");
			return; 
		} 
		
		var url = that.URL_FETCH_STRINGS + "ath" + lang + "-utf8.asp";
		Debug.ConsoleOut("Fetching strings: url=" + url);
	
		req.open("GET", url, true);
		req.onreadystatechange = function() { Comms.OnFetchStringsReadyStateChange(req); };
		req.send("");
	};
	
	/*sfh
	******************************************************************************
	
		Function:		SetForceIFRAMEComms
	
		Desc:			Forces the comms library to use an IFRAME for 
						communication instead of XMLHttpRequest objects.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.SetForceIFRAMEComms = function(bForce)
	{
		bForce_IFrame_Request = bForce ? true : false;
	};
	
	/*sfh
	******************************************************************************
	
		Function:		CreateXMLHttpRequest
	
		Desc:			Creates a new XMLHttpRequest object.
	
		Returns:		XMLHttpRequest
	
	******************************************************************************
	efh*/
	
	this.CreateXMLHttpRequest = function()
	{
		var req = null;
		if (!bForce_IFrame_Request)
		{
			if( !this.bArchivePlayer)
			{
				try { if(!req) { req = new XMLHttpRequest(); }} catch(e) { req=null; }
				try { if(!req) { req = new ActiveXObject("Msxml2.XMLHTTP.5.0"); }} catch (e) { req=null; }
				try { if(!req) { req = new ActiveXObject("Msxml2.XMLHTTP.4.0"); }} catch (e) { req=null; }
				try { if(!req) { req = new ActiveXObject("Msxml2.XMLHTTP.3.0"); }} catch (e) { req=null; }
				try { if(!req) { req = new ActiveXObject("Msxml2.XMLHTTP"); }} catch (e) { req=null; }
				try { if(!req) { req = new ActiveXObject("Microsoft.XMLHTTP"); }} catch (e) { req=null; }
			}
			else
			{
				try { if(!req) { req = new ActiveXObject("Msxml2.XMLHTTP.5.0"); }} catch (e) { req=null; }
				try { if(!req) { req = new ActiveXObject("Msxml2.XMLHTTP.4.0"); }} catch (e) { req=null; }
				try { if(!req) { req = new ActiveXObject("Msxml2.XMLHTTP.3.0"); }} catch (e) { req=null; }
				try { if(!req) { req = new ActiveXObject("Msxml2.XMLHTTP"); }} catch (e) { req=null; }
				try { if(!req) { req = new ActiveXObject("Microsoft.XMLHTTP"); }} catch (e) { req=null; }
				try { if(!req) { req = new XMLHttpRequest(); }} catch(e) { req=null; }
			}
		}
		try { if(!req) { req = new IFRAMERequest(); }} catch(e) { req=null; }
	
		if (!req)
		{
			Debug.ConsoleOut("FAILED to create XMLHttpRequest object!");
			window.alert("FAILED to create XMLHttpRequest object!");
		}
		else
		{
			//Debug.ConsoleOut("Success: XMLHttpRequest object = " + req + " :: readyState = " + req.readyState);
		}
		return req;
	};

} // End of CommsLibObject

//
//
// 		IFRAMERequest  object
//
//

/*sfh
******************************************************************************

	Function:		IFRAMERequest

	Desc:			Creates an object that simulates the XMLHttpRequest 
					object using an IFRAME.

	Returns:		IFRAMERequest

******************************************************************************
efh*/

IFRAMERequest.prototype.readyState = 0;
IFRAMERequest.prototype.onreadystatechange = null;
IFRAMERequest.prototype.status = 0;
IFRAMERequest.prototype.statusText = "";
IFRAMERequest.prototype.responseText = "";
IFRAMERequest.prototype.responseXML = "";

function IFRAMERequest()
{
	Debug.ConsoleOut("Creating IFRAMERequest object - simulating XMLHTTPRequest");

	if (!IFRAMERequest.nextID) { IFRAMERequest.nextID = 1; }

	this.method = "";
	this.url = "";
	this.async = true;
	this.user = "";
	this.password = "";	
	
	this.iframe = null;
}

//
// Abort function from XMLHTTPRequest object
//
IFRAMERequest.prototype.abort = function() 
{
	this.readyState = 4;
	this.statusText = "aborted";
	if (this.onreadystatechange) { this.onreadystatechange(); }
};

//
// getAllResponseHeaders function from XMLHTTPRequest object
//
IFRAMERequest.prototype.getAllResponseHeaders = function() 
{
	return "";
};

//
// getResponseHeader function from XMLHTTPRequest object
//
IFRAMERequest.prototype.getResponseHeader = function(headerName) 
{
	return "";
};

//
// setResponseHeader function from XMLHTTPRequest object
//
IFRAMERequest.prototype.setResponseHeader = function(name, value) 
{
	// Not supported.
};

//
// Sets the URL and other settings to be used when downloading a resource.
//
IFRAMERequest.prototype.open = function(method, url, async, user, password) 
{
	if (!async) { Debug.ConsoleOut("IFRAMERequest - !! WARNING !! - async false is not supported."); }
	if (method != "get" && method != "GET") { Debug.ConsoleOut("IFRAMERequest - !! WARNING !! - POST is not supported."); }

	var bNotify = this.readyState !== 0 ? true : false;
	this.readyState = 0;
	this.method = method;
	this.url = url;
	this.async = async;
	this.user = user;
	this.password = password;
	if (bNotify && this.onreadystatechange) { this.onreadystatechange(); }
};

//
// Performs the download
//
IFRAMERequest.prototype.send = function(body) 
{
	if (!this.iframe)
	{
		// Create a new IFRAME that can download the url
		this.iframe = document.createElement("iframe");
		this.iframe.id = "iframe" + IFRAMERequest.nextID;
		IFRAMERequest.nextID = IFRAMERequest.nextID + 1;
		this.iframe.iframeRequest = this;
		Debug.ConsoleOut("IFRAME created: id=" + this.iframe.id);
		
		// Hide the frame
		this.iframe.style.position = "absolute";
		this.iframe.style.top = "10px";
		this.iframe.style.left = "10px";
		this.iframe.style.width = "900px";
		this.iframe.style.height = "200px";
		this.iframe.style.visibility = "hidden";
		
		document.body.appendChild(this.iframe);
	}

	clearIFRAMETimers(this.iframe);

	this.iframe.onload = handleOnLoad;
	this.iframe.onabort = handleOnError;
	this.iframe.onerror = handleOnError;

	this.iframe.src = this.url;  // Set the iframe src to new url.
	this.readyState = 1;
	if (this.onreadystatechange) { this.onreadystatechange(); }
	
	if (!IFRAMERequest.supportsOnload)
	{
		this.timerCheck = window.setTimeout("handleIFRAMECheck('" + this.iframe.id + "');", 500);
	}
	this.timerTimeout = window.setTimeout("handleIFRAMETimeout('" + this.iframe.id + "');", 35000);
};

/*sfh
******************************************************************************

	Function:		handleOnLoad

	Desc:			Called when the IFRAME is finished downloading a URL,
					but only in browsers that support the "onload" event
					on IFRAMES (Mozilla).

	Returns:		void

******************************************************************************
efh*/

function handleOnLoad(evt)
{
	if (Debug.IsDebug()) { Debug.ConsoleOut("-handleOnLoad-"); }

	var event = D2E.CreateDOM2Event(evt, window.event);
	var iframe = event.target;
	
	if (iframe && iframe.id)
	{
		// Call the real handler.
		IFRAMERequest.supportsOnload = true;
		handleIFRAMELoad(iframe.id);
	}
}

/*sfh
******************************************************************************

	Function:		handleOnError

	Desc:			Called when the IFRAME encounters an error downloading 
					a URL, but only in browsers that support the "onload" 
					event on IFRAMES (Mozilla).

	Returns:		void

******************************************************************************
efh*/

function handleOnError(evt)
{
	if (Debug.IsDebug()) { Debug.ConsoleOut("-handleOnError-");	}

	var event = D2E.CreateDOM2Event(evt, window.event);
	var iframe = event.target;

	if (iframe && iframe.id)
	{
		// Call the real handler.
		handleIFRAMEError(iframe.id);
	}
}

/*sfh
******************************************************************************

	Function:		handleIFRAMELoad

	Desc:			Called when the IFRAME is finished downloading a URL.

	Returns:		void

******************************************************************************
efh*/


function handleIFRAMELoad(frameID)
{
	if (Debug.IsDebug()) { Debug.ConsoleOut("handleIFRAMELoad - id= " + frameID); }

	var iframe = document.getElementById(frameID);
	var req = iframe.iframeRequest;

	// Cancel the timers
	clearIFRAMETimers(iframe);

	req.readyState = 4;
	req.status = 200;
	req.statusText = "200 - OK";
	req.responseText = getIFRAMEInnerHTML(iframe);
	if (req.onreadystatechange) { req.onreadystatechange(); }
}

/*sfh
******************************************************************************

	Function:		handleIFRAMEError

	Desc:			Called when the IFRAME encounters an error while 
	                downloading a url.

	Returns:		void

******************************************************************************
efh*/

function handleIFRAMEError(frameID) 
{
	var iframe = document.getElementById(frameID);
	var req = iframe.iframeRequest;

	// Cancel the timers
	clearIFRAMETimers(iframe);

	if (Debug.IsDebug()) { Debug.ConsoleOut("handleIFRAMEError - id = " + frameID + " - url = " + req.url); }

	req.readyState = 4;
	req.status = 500;
	req.statusText = "500 - Undetermined Error";
	req.responseText = "";
	if (req.onreadystatechange) { req.onreadystatechange(); }
}

/*sfh
******************************************************************************

	Function:		handleIFRAMECheck

	Desc:			Called repeatedly to check if the IFRAME is finished
					downloading the URL.

	Returns:		void

******************************************************************************
efh*/

function handleIFRAMECheck(frameID)
{
	var iframe = document.getElementById(frameID);
	var req = iframe.iframeRequest;
	var inner = getIFRAMEInnerHTML(iframe);
	if (!inner)
	{
		// Not done yet...
		if (Debug.IsDebug()) { Debug.ConsoleOut("handleIFRAMECheck - id=" + frameID + "  ... not done yet..."); }
		req.timerCheck = window.setTimeout("handleIFRAMECheck('" + frameID + "');", 500);
	}
	else
	{
		// Download finished
		if (Debug.IsDebug()) { Debug.ConsoleOut("handleIFRAMECheck - id=" + frameID + "  ... download complete"); }
		handleIFRAMELoad(frameID);
	}
}

/*sfh
******************************************************************************

	Function:		handleIFRAMETimeout

	Desc:			Called when the IFRAME times out while downloading a URL.

	Returns:		void

******************************************************************************
efh*/

function handleIFRAMETimeout(frameID)
{
	if (Debug.IsDebug()) { Debug.ConsoleOut("handleIFRAMETimeout - id=" + frameID + "  ... download timed out!"); }
	handleIFRAMEError(frameID);
}

/*sfh
******************************************************************************

	Function:		clearIFRAMETimers

	Desc:			Clears (cancels) all timers for the specified IFRAME.

	Returns:		void

******************************************************************************
efh*/

function clearIFRAMETimers(iframe)
{
	var req = iframe.iframeRequest;
	if (req.timerCheck)
	{
		window.clearTimeout(req.timerCheck);
		req.timerCheck = 0;
	}
	if (req.timerTimeout) 
	{
		window.clearTimeout(req.timerTimeout);
		req.timerTimeout = 0;
	}
}

/*sfh
******************************************************************************

	Function:		getIFRAMEDoc

	Desc:			Returns the document object for the specified IFRAME.

	Returns:		Document (or null)

******************************************************************************
efh*/

function getIFRAMEDoc(iframe)
{
	var theDoc = null;

	if (iframe.contentDocument)  // NS6
	{
		theDoc = iframe.contentDocument;
	}
	else if (iframe.contentWindow) // IE 5.5 & IE 6
	{
		theDoc = iframe.contentWindow.document;
	}
	else if (iframe.document)
	{
		theDoc = iframe.document;
	}
	return theDoc;
}

/*sfh
******************************************************************************

	Function:		getIFRAMEInnerHTML

	Desc:			Returns the innerHTML for the specified IFRAME.

	Returns:		string (or null)

******************************************************************************
efh*/

function getIFRAMEInnerHTML(iframe)
{
	var theDoc = getIFRAMEDoc(iframe);
	var inner = null;
	try
	{
		inner = theDoc.body.innerHTML;		
	}
	catch(e)
	{
		inner = null;
	}
	return inner;
}

// End of file
