//
// Method for getting the DebugTools object
//

function GetDebugObject()
{
	if (!GetDebugObject.debug)
	{
		GetDebugObject.debug = new DebugObject();
	}
	return GetDebugObject.debug;
}

var Debug = GetDebugObject();

//
// Object with methods useful for debug
//
 
function DebugObject()
{
	var that = this;
	var ConsoleOutWnd = null;
	var m_bForceDebug = false;


	/*sfh
	******************************************************************************
	
		Function:		IsDebug
	
		Desc:			Returns true if debug mode is on.
	
		Returns:		bool
	
	******************************************************************************
	efh*/
	
	this.IsDebug = function()
	{
		var myLoc = window.location;
		var i = myLoc.href.indexOf("debug=1");
		return (i > 0 || m_bForceDebug) ? true : false;
	};
	
	/*sfh
	******************************************************************************
	
		Function:		SetForceDebug
	
		Desc:			Allows the debug on/off state to be overridden even if
						"debug=1" does not appear in the URL line.
	
		Returns:		bool
	
	******************************************************************************
	efh*/
	
	this.SetForceDebug = function(bDebug)
	{
		m_bForceDebug = bDebug;

		// If the debug window was closed, then reset so it will be
		// opened again on next "ConsoleOut".
		if (ConsoleOutWnd && ConsoleOutWnd.closed)
		{
			ConsoleOutWnd = null;
		}
	};

	/*sfh
	******************************************************************************
	
		Function:		ConsoleOut
	
		Desc:			Outputs the string to the debug console. A console window 
		                is automatically created.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.ConsoleOut = function(str)
	{
		try
		{
			ConsoleOutImpl(str);
		}
		catch (e) { }  
	}; 
	
	/*sfh
	******************************************************************************
	
		Function:		ConsoleOutImpl
	
		Desc:			Implementation for ConsoleOut
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function ConsoleOutImpl(str)
	{
		if (!that.IsDebug())
		{
			return;
		}
		
		var strCmdClear = "$*!!Clear!!*$";
	
		if (!ConsoleOutWnd)
		{
			var strJS = "if (opener.Debug.ConsoleOut != null) opener.Debug.ConsoleOut('" + strCmdClear + "')";
	
			var strHTML = "<html><head><title>Console</title></head>";
			strHTML = strHTML + "<body><span id='ConsoleDiv' style='white-space: nowrap;'></span>";
			strHTML = strHTML + "<p><a href=\"javascript:" + strJS + "\">Clear</a></p>";
			strHTML = strHTML + "</body></html>";
	
			// Create the window, plus DIV.
			ConsoleOutWnd = OpenConsoleWindow("Console");
			ConsoleOutWnd.document.write(strHTML);
			ConsoleOutWnd.document.close();
		}
	
		if (ConsoleOutWnd && !ConsoleOutWnd.closed)
		{
			var elem = ConsoleOutWnd.document.getElementById("ConsoleDiv");
			if (elem)
			{
				// Check if we're getting the Clear command.
				if (str == strCmdClear)
				{
					while (elem.firstChild)
					{
						elem.removeChild(elem.firstChild); 
					}
				}
				else  // Just append the new string.
				{
					// Add a timestamp.
					var now = new Date();
					var strTS = ("00" + now.getHours()).slice(-2);
					strTS = strTS + ":" + ("00" + now.getMinutes()).slice(-2);
					strTS = strTS + ":" + ("00" + now.getSeconds()).slice(-2);
					strTS = strTS + ":" + ("000" + now.getMilliseconds()).slice(-3);
					str = strTS + " - " + str;

					elem.appendChild(ConsoleOutWnd.document.createTextNode(str));
					elem.appendChild(ConsoleOutWnd.document.createElement("br"));
					ConsoleOutWnd.scrollTo(0,100000000);
				}
			}
		}
	}
	
	/*sfh
	******************************************************************************
	
		Function:		ConsoleOutProps
	
		Desc:			Outputs the properties of the specified object to the
						debug console.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.ConsoleOutProps = function(obj)
	{
		that.ConsoleOut("---------------------");
		that.ConsoleOut("--- Object: " + obj);
		if (obj)
		{
			for (var p in obj) 
			{
				that.ConsoleOut("--- " + p + " = " + obj[p]);
			}
		}
		that.ConsoleOut("---------------------");
	};

	/*sfh
	******************************************************************************
	
		Function:		ConsoleOutException
	
		Desc:			Outputs the properties of the specified exception object 
						to the debug console.
	
		Returns:		void
	
	******************************************************************************
	efh*/

	this.ConsoleOutException = function(e)
	{
		if (e !== null)
		{
			that.ConsoleOut("-------------------------------");
			that.ConsoleOut("---- EXCEPTION CAUGHT: " + e.description);
			that.ConsoleOut("-------------------------------");
			that.ConsoleOut("---- fileName        : " + e.fileName);
			that.ConsoleOut("---- lineNumber      : " + e.lineNumber);
			that.ConsoleOut("---- message         : " + e.message);
			that.ConsoleOut("---- name            : " + e.name);
			that.ConsoleOut("---- number          : " + e.number);			
			that.ConsoleOut("---- stack           : " + e.stack);			
			that.ConsoleOut("-------------------------------");
		}
	};
	
	/*sfh
	******************************************************************************
	
		Function:		OpenConsoleWindow
	
		Desc:			Creates a new window suitable for debug output.
	
		Returns:		window
	
	******************************************************************************
	efh*/
	
	function OpenConsoleWindow(strTitle)
	{
		var strFeatures = "height=200,width=500,scrollbars=yes,dependent=yes,resizable=yes,";
		strFeatures = strFeatures + "directories=no,menubar=no,status=no,toolbar=no";
	
		return window.open("", strTitle, strFeatures, false);
	}

} // End of DebugObject

// End of file