/**sfh
******************************************************************************

	DOM2Event
	~~~~~~~~~
		 
		  Forces all supported browsers to emulate a single event model
		  based on DOM2 events. As much as possible buggy event properties
		  are corrected (Opera's clientX for example).
		  
		  Keyboard events are supported even though they are not supported
		  by DOM2.

		  Usage: 
		  
		  		Add event listeners as per normal, for example 
		  		
		  			onmousemove=myHandler;
		  			
		  		When an event is handled, convert to a DOM2Event using the
		  		createDOM2Event method. For example
		  		
		  			var D2E = GetDOM2EventObject();
		  			function myHandler (evt) 
		  			{ 
		  				event = D2E.createDOM2Event(evt, window.event); 
		  			    // do more stuff ... 
		  			}

******************************************************************************
efh*/

function GetDOM2EventObject()
{
	if (!GetDOM2EventObject.D2E)
	{
		GetDOM2EventObject.D2E = new DOM2EventObject();
	}
	return GetDOM2EventObject.D2E;
}

var Debug = GetDebugObject();
var D2E = GetDOM2EventObject();

//
// DOM2EventObject
//

function DOM2EventObject()
{
	//
	// Statics
	//
	//this.s_PHASE_CAPTURE = 1;
	//this.s_PHASE_AT_TARGET = 2;
	//this.s_PHASE_BUBBLE = 3;

	// Standard mouse button identifiers for javascript events		
	this.s_MOUSEBUTTON_LEFT = 0;
	this.s_MOUSEBUTTON_MIDDLE = 1;
	this.s_MOUSEBUTTON_RIGHT = 2;
	
	// Windows API constants for mouse button state modifiers (KB events)
	this.s_MK_LBUTTON = 0x0001;
	this.s_MK_RBUTTON = 0x0002;
	this.s_MK_SHIFT = 0x0004;
	this.s_MK_CONTROL = 0x0008;
	this.s_MK_MBUTTON = 0x0010;

	// Windows API constants for mouse events
	this.s_EVTID_MOUSE_MOVE = 301;
	this.s_EVTID_MOUSE_LEFT_DOWN = 302;
	this.s_EVTID_MOUSE_LEFT_UP = 303;
	this.s_EVTID_MOUSE_LEFT_DBLCLICK = 304;
	this.s_EVTID_MOUSE_MIDDLE_DOWN = 305;
	this.s_EVTID_MOUSE_MIDDLE_UP = 306;
	this.s_EVTID_MOUSE_MIDDLE_DBLCLICK = 307;
	this.s_EVTID_MOUSE_RIGHT_DOWN = 308;
	this.s_EVTID_MOUSE_RIGHT_UP = 309;
	this.s_EVTID_MOUSE_RIGHT_DBLCLICK = 310;
	
	/*sfh
	******************************************************************************
	
		Function:		CreateDOM2Event
	
		Desc:			Converts from a proprietary event to a DOM2 event.
	
		Returns:		DOM2Event
	
	******************************************************************************
	efh*/
	
	this.CreateDOM2Event = function(evt, ieevt)
	{
		var srcEvt = GetSrcEvent(evt, ieevt);
		if (!srcEvt)
		{
			return null;
		}
	
		var event = new Object();
	
		// Common properties (for all events)
		event.orig_event = srcEvt;
		event.bubbles = IsBubbler(srcEvt);
		event.cancelable = IsCancelable(srcEvt);
		event.currentTarget = srcEvt.currentTarget;
		event.target = GetTarget(srcEvt);
		//event.eventPhase = null;  // Not needed.  GetEventPhase(srcEvt, event.currentTarget, event.target);
		event.type = srcEvt.type;
	
		// Mouse event properties
		event.altKey = (srcEvt.altKey) ? srcEvt.altKey : false;
		event.button = GetMouseButton(srcEvt);
		event.clientX = srcEvt.clientX;			// TODO: Handle scrollbar for non-IE
		event.clientY = srcEvt.clientY;			// TODO: Handle scrollbar for non-IE
		event.pageX = GetPageX(srcEvt);
		event.pageY = GetPageY(srcEvt);
		event.ctrlKey = (srcEvt.ctrlKey) ? srcEvt.ctrlKey : false;
		event.detail = (srcEvt.detail) ? srcEvt.detail : 0;
		event.metaKey = (srcEvt.metaKey) ? srcEvt.metaKey : false;
		// event.relatedTarget = null; // Not needed
		event.screenX = (srcEvt.screenX) ? srcEvt.screenX : 0;
		event.screenY = (srcEvt.screenY) ? srcEvt.screenY : 0;
		event.shiftKey = (srcEvt.shiftKey) ? srcEvt.shiftKey : false; 
		
		// Keyboard event property (not in DOM2 spec!) 
		event.keyCode = (srcEvt.keyCode) ? srcEvt.keyCode : 0; 
	
		return event;
	};
	
	/*sfh
	******************************************************************************
	
		Function:		GetSrcEvent
	
		Desc:			Returns the correct source event object
	
		Returns:		Event
	
	******************************************************************************
	efh*/
	
	function GetSrcEvent(evt, ieevt)
	{
		var src = null;
		if (ieevt)
		{
			src = ieevt;
		}
		else if (evt)
		{
			src = evt;
		}
		return src;
	}
	
	/*sfh
	******************************************************************************
	
		Function:		IsBubbler
	
		Desc:			Returns true if the specified event bubbles up.
	
		Returns:		boolean
	
	******************************************************************************
	efh*/
	
	function IsBubbler(evt)
	{
		var bRet = true;
		if (evt.bubbles)
		{
			bRet = evt.bubbles;
		}
		else if (evt.type == 'blur' || evt.type == 'focus' || 
		         evt.type == 'load' || evt.type == 'unload')
		{
	 		bRet = false;
		}
		return bRet;
	}
	
	/*sfh
	******************************************************************************
	
		Function:		IsCancelable
	
		Desc:			Returns true if the specified event can be cancelled.
	
		Returns:		boolean
	
	******************************************************************************
	efh*/
	
	function IsCancelable(evt)
	{
		var bRet = false;
		if (evt.cancelable)
		{
			bRet = evt.cancelable;
		}
		else if (evt.type == "click" || evt.type == "mousedown" || 
				 evt.type == "mouseout" || evt.type == "mouseover" || 
				 evt.type == "mouseup" || evt.type == "submit")
		{
			bRet = true;
		}
		return bRet;
	}
	
	/*sfh
	******************************************************************************
	
		Function:		GetTarget
	
		Desc:			Returns the node to which the event was originally 
						dispatched.
	
		Returns:		node/element
	
	******************************************************************************
	efh*/
	
	function GetTarget(evt)
	{
		var target = null;
		if (evt && evt.target)
		{
			target = evt.target;
		}
		else if (evt && evt.srcElement)
		{
			target = evt.srcElement;
		}
		else if (evt && evt.currentTarget)
		{
			target = evt.currentTarget;
		}
		return target;
	}
	
	/*sfh
	******************************************************************************
	
		Function:		GetEventPhase
	
		Desc:			Returns an integer indicating the current phase of the 
						event flow.
							1 = capture
							2 = at target
							3 = bubble
	
		Returns:		int
	
	******************************************************************************
	efh*/

/* Not needed	
	function GetEventPhase(evt, currentTarget, target)
	{
		var iPhase = this.s_PHASE_BUBBLE;
		if (evt.eventPhase)
		{
			iPhase = evt.eventPhase;
		}
		else if (currentTarget == target)
		{
			iPhase = this.s_PHASE_AT_TARGET;
		}
		return iPhase;
	}
*/
	
	/**sfh
	******************************************************************************
	
		Function:		GetMouseButton
	
		Desc:			Returns an integer indicating which mouse button was
						clicked.
							0 = left
							1 = middle
							2 = right
							
						Here is a list of the values returned for different 
						browsers:
						
							event.button:
							_____________________________________________________
							| IE  			| left=1, middle=4, right=2 		|
							| Konqueror		| 									|
							|---------------------------------------------------|
							| Netscape 4	| all undefined						|	
							|---------------------------------------------------|
							| Gecko 0.9		| left=1, middle=2, right=3			|	
							|---------------------------------------------------|
							| Opera 7		| left=1, middle=3, right=2			|
							|---------------------------------------------------|
							| Gecko 1.0+	| left=0, middle=1, right=2			|
							| Opera 9		|									|
							| Windows Safari|			(most standard)			|
							|---------------------------------------------------|
							| Mac Safari	| left=1							|
							|___________________________________________________|

							event.which:
							_____________________________________________________
							| IE  			| all undefined				 		|
							|---------------------------------------------------|
							| Gecko 0.9		| left=1, middle=2, right=2			|	
							|---------------------------------------------------|
							| Opera 7		| left=1, middle=3, right=3			|
							|---------------------------------------------------|
							| Konqueror		| left=1, middle=2, right=3			|	
							| Netscape 4	| 									|	
							| Gecko 1.0+	| 									|
							| Opera 9		|									|
							| Windows Safari|			 						|
							|---------------------------------------------------|
							| Mac Safari	| left=1							|
							|___________________________________________________|
	
		Returns:		int
	
	******************************************************************************
	efh*/
	
	function GetMouseButton(evt)
	{
		var button = D2E.s_MOUSEBUTTON_LEFT;

		// This simple algorithm makes use of the fact that IE doesn't support "event.which"
		// and the other browsers have similar values for "event.which".
		if (!evt.which)
		{
       		// This must be IE
			button = (evt.button < 2) ? D2E.s_MOUSEBUTTON_LEFT : ((evt.button == 4) ? D2E.s_MOUSEBUTTON_MIDDLE : D2E.s_MOUSEBUTTON_RIGHT);
		}
		else
		{
			// All other browsers support "which" and they are all close in meaning
			button = (evt.which < 2) ? D2E.s_MOUSEBUTTON_LEFT : ((evt.which == 2) ? D2E.s_MOUSEBUTTON_MIDDLE : D2E.s_MOUSEBUTTON_RIGHT);
		}
		return button;
	}
	
	/*sfh
	******************************************************************************
	
		Function:		GetPageX
	
		Desc:			Returns the mouse events coodinates relative to the
						document or page.
	
		Returns:		int
	
	******************************************************************************
	efh*/
	
	function GetPageX(evt)
	{
		var pageX = 0;
		var ua = navigator.userAgent.toLowerCase(); 
	
		if (evt.pageX)
		{
			// Handles Mozilla, Safari, Opera 8+, Netscape, or any browser that
			// supports pageX.
			pageX = evt.pageX;
		}
		else
		{
			var isOpera = (ua.indexOf("opera") != -1) ? true : false;
			var isIE = (ua.indexOf("msie") != -1) ? true : false;
			var isKonq = (ua.indexOf("konqueror") != -1) ? true : false;
			
	   		if (isOpera || isKonq)
	   		{
	   			// clientX is relative to the document in Opera 6&7, and Konq.
	   			pageX = evt.clientX;
	   		}
			else if (isIE) 
			{
				var docElem = document.documentElement;
				if (docElem && docElem.scrollLeft)
				{
		        	pageX = evt.clientX + docElem.scrollLeft;
		      	}
		      	else if (document.body && document.body.scrollLeft)
		      	{
		        	pageX = evt.clientX + document.body.scrollLeft;
		      	}
		      	else
		      	{
		      		pageX = evt.clientX;
		      	}
	   		}
		}
		return pageX;
	}
	
	/*sfh
	******************************************************************************
	
		Function:		GetPageY
	
		Desc:			Returns the mouse events coodinates relative to the
						document or page.
	
		Returns:		int
	
	******************************************************************************
	efh*/
	
	function GetPageY(evt)
	{
		var pageY = 0;
		var ua = navigator.userAgent.toLowerCase(); 
	
		if (evt.pageY)
		{
			// Handles Mozilla, Safari, Opera 8+, Netscape.
			pageY = evt.pageY;
		}
		else
		{
			var isOpera = (ua.indexOf("opera") != -1) ? true : false;
			var isIE = (ua.indexOf("msie") != -1) ? true : false;
			var isKonq = (ua.indexOf("konqueror") != -1) ? true : false;
	
	   		if (isOpera || isKonq)
	   		{
	   			// clientX is relative to the document in Opera 6&7, and Konq.
	   			pageY = evt.clientY;
	   		}
			else if (isIE) 
			{
				var docElem = document.documentElement;
				if (docElem && docElem.scrollTop > 0)
				{
		        	pageY = evt.clientY + docElem.scrollTop;
		      	}
		      	else if (document.body && document.body.scrollTop > 0)
		      	{
		        	pageY = evt.clientY + document.body.scrollTop;
		      	}
		      	else
		      	{
		      		pageY = evt.clientY;
		      	}
	   		}
		}
		return pageY;
	}
	
	/*sfh
	******************************************************************************
	
		Function:		CancelBubbling
	
		Desc:			Stops the event from bubbling up the element tree, and
						sets the event's return value to false to block the
						browser's default behaviour.
	
						Pass the DOM/Mozilla event object and the IE window.event
						object.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.CancelBubbling = function(evt)
	{
		if (evt && evt.orig_event)
		{
			// IE
			evt.orig_event.cancelBubble = true; 
			evt.orig_event.returnValue = false; 
			try { evt.orig_event.keyCode = 0; } catch(e) { Debug.ConsoleOut("CancelBubbling -- cannot reset keyCode"); }
			
			// DOM2
			if (evt.orig_event.preventDefault) { evt.orig_event.preventDefault(); }
			if (evt.orig_event.stopPropagation) { evt.orig_event.stopPropagation(); }
		}
	};
	
	/*sfh
	******************************************************************************
	
		Function:		GetWindowsMouseEventID
	
		Desc:			Returns the Windows WM_XXX style ID for the specified
						DOM2Event.
	
						The conversion is done by checking the button states and
						mouse event type.
	
		Returns:		int
	
	******************************************************************************
	efh*/
	
	this.GetWindowsMouseEventID = function(evt)
	{
		var id = 0;
		
		if (evt)
		{
			var button = evt.button;
			switch(evt.type)
			{
				case "mousemove": 
					id = this.s_EVTID_MOUSE_MOVE;
					break;
				case "mousedown":
					if (button == this.s_MOUSEBUTTON_LEFT) { id = this.s_EVTID_MOUSE_LEFT_DOWN; }
					else if (button == this.s_MOUSEBUTTON_MIDDLE)  { id = this.s_EVTID_MOUSE_MIDDLE_DOWN; }
					else if (button == this.s_MOUSEBUTTON_RIGHT) { id = this.s_EVTID_MOUSE_RIGHT_DOWN; }
					break;
				case "mouseup":
					if (button == this.s_MOUSEBUTTON_LEFT) { id = this.s_EVTID_MOUSE_LEFT_UP; }
					else if (button == this.s_MOUSEBUTTON_MIDDLE)  { id = this.s_EVTID_MOUSE_MIDDLE_UP; }
					else if (button == this.s_MOUSEBUTTON_RIGHT) { id = this.s_EVTID_MOUSE_RIGHT_UP; }
					break;
				case "dblclick":
					if (button == this.s_MOUSEBUTTON_LEFT) { id = this.s_EVTID_MOUSE_LEFT_DBLCLICK; }
					else if (button == this.s_MOUSEBUTTON_MIDDLE)  { id = this.s_EVTID_MOUSE_MIDDLE_DBLCLICK; }
					else if (button == this.s_MOUSEBUTTON_RIGHT) { id = this.s_EVTID_MOUSE_RIGHT_DBLCLICK; }
					break;
				default:
					Debug.ConsoleOut("DOM2Event.GetWindowsMouseEventID -- not handling event type=" + evt.type);
					break;
			}	
		}
		return id;
	};
	
	/*sfh
	******************************************************************************
	
		Function:		GetWindowsKeyStateMask
	
		Desc:			Returns the Windows MK_XXX style bitflags representing
						the state of mouse buttons and Shift/Control keys
						at the time of the mouse event.
	
						The conversion is done by checking the button states and
						mouse event type.
	
		Returns:		int
	
	******************************************************************************
	efh*/
	
	this.GetWindowsKeyStateMask = function(evt)
	{
		var flags = 0;

		if (evt.button == this.s_MOUSEBUTTON_LEFT) { flags |= this.s_MK_LBUTTON; }
		if (evt.button == this.s_MOUSEBUTTON_MIDDLE) { flags |= this.s_MK_MBUTTON; }
		if (evt.button == this.s_MOUSEBUTTON_RIGHT) { flags |= this.s_MK_RBUTTON; }
		
		if (evt.shiftKey) { flags |= this.s_MK_SHIFT; }
		if (evt.ctrlKey) { flags |= this.s_MK_CONTROL; }
	
		return flags;
	};

} // End of DOM2EventObject

// End of file