/*sfh
******************************************************************************


 Engine API - Functions intended to be called from other files are 
 ~~~~~~~~~~   prefixed with "Eng"

	APIs to be called externally are prefixed with "Eng". See list below. 

******************************************************************************
efh*/

function GetEngineObject()
{ 
	if (!GetEngineObject.theEngine)
	{
		GetEngineObject.theEngine = new EngineObject();
	}
	return GetEngineObject.theEngine; 
}

var Debug = GetDebugObject();
var Comms = GetCommsObject();
var Engine = GetEngineObject();

//
// EngineObject
//

function EngineObject()
{
	//
	// Statics
	//
	var s_IMAGE_LOAD_TIMEOUT = 15000; 		// Time alloted to download an image before retrying
	var s_IMAGE_LOAD_MAX_RETRIES = 3; 		// Max number of image download retries
	var s_BATCH_RENDER_TIMEOUT = 5000;		// Max time to delay rendering of a batch of tiles
	var s_TOOLTIP_LINE_MAX_CHARS = 12;		// Max number of characters per tooltip line
	var s_POINT_FIVE = 1/2;					// Hack to get around validator warning
	var s_SESSION_TYPE_APPSHARE = "1";		// Session type, provided by server
	var s_SESSION_TYPE_SLIDESHOW = "2";
	var s_STATUS_NONE = 0;					// Download status
	var s_STATUS_DOWNLOADING = 1;
	var s_STATUS_DONE = 2;
	
	// Session info
	var m_session = new Object();
	m_session.sesstype = s_SESSION_TYPE_APPSHARE;		// appshare session type, default is regular appsharing
	m_session.bSharing = false;							// True if something is currently shared
	
	// Objects
	var m_div = null;									// The div element containing the tiles
	var m_tileMap = null;           					// Tile Map
	var m_cursor = null;								// The mouse cursor
	
	// Metrics
	var m_metrics = new Object();
	m_metrics.screenWidth = 0;				// Width of moderator's screen
	m_metrics.screenHeight = 0;				// Height of moderator's screen
	m_metrics.numXTiles = 0;  				// Number of tiles across
	m_metrics.numYTiles = 0; 				// Number of tiles vertical
	m_metrics.tileWidth = 0;				// Width of each tile
	m_metrics.tileHeight = 0;				// Height of each tile
	m_metrics.cursorPosX = 0;				// Current cursor position
	m_metrics.cursorPosY = 0;
	m_metrics.offsetX = 0;					// Offset of tile div 
	m_metrics.offsetY = 0;
	m_metrics.sharedRectWidth = 0;			// Width and height of the shared area
	m_metrics.sharedRectHeight = 0;		
	m_metrics.zoom = "100";					// Current ZOOM level
	m_metrics.zoomActual = "100";
	m_metrics.bAutoscroll = false;			// Determines if auto-scroll is enabled

	// Batch updates
	var m_batch = new Object();
	m_batch.id = 0;							// Current batch id for grouping tile updates.
	m_batch.timer = 0;						// Timer used to force batch paint after elapsed time
	m_batch.queue = new Array();			// Queue of tile ids which have new images downloaded

	//
	//
	// EXTERNAL API
	//
	//
	this.EngEngineInit = function(zoom, bAutoscroll) { EngineInit(zoom, bAutoscroll); };
	this.EngSetClientID = function(cid) { SetClientID(cid); };
	this.EngSetSessType = function(sesstype) { SetSessType(sesstype); };
	this.EngBuildTileMap = function(numXTiles, numYTiles, tileWidth, tileHeight, screenWidth, screenHeight) { BuildTileMap(numXTiles, numYTiles, tileWidth, tileHeight, screenWidth, screenHeight); };
	this.EngSetZoom = function(iZoom) { SetZoom(iZoom); };
	this.EngGetZoom = function() { return m_metrics.zoom; };
	this.EngGetActualZoom = function() { return m_metrics.zoomActual; };
	this.EngIsZoomSupported = function() { return IsZoomSupported(); };
	this.EngSetAutoScroll = function(bAutoScroll) { SetAutoScroll(bAutoScroll); };
	this.EngMarkTileDirty = function(iTileID, iBatchID, url, bLastInBatch) { MarkTileDirty(iTileID, iBatchID, url, bLastInBatch); };
	this.EngSetTileCropped = function(iTileID, urlPNG, pngX, pngY) { SetTileCropped(iTileID, urlPNG, pngX, pngY); };
	this.EngMarkCursorImageDirty = function(url) { this.MarkCursorImageDirty(url); };
	this.EngMoveCursor = function(x,y) { MoveCursor(x,y); };
	this.EngSetCursorCropped = function(urlImg, imgX, imgY, cursorWidth, cursorHeight) { SetCursorCropped(urlImg, imgX, imgY, cursorWidth, cursorHeight); };
	this.EngNotifySharing = function(bSharing) { NotifySharing(bSharing); };
	this.EngShowTooltip = function(bShow, txt) { ShowTooltip(bShow, txt); };
	this.EngSetLocalCursor = function(strCursor) { SetLocalCursor(strCursor); };
	this.EngSetScreenOffset = function(x,y) { SetScreenOffset(x,y); };
	this.EngSetSharedRect = function(left,top,width,height) { SetSharedRect(left,top,width,height); };
	this.EngGetPageWidth = function() { return GetPageWidth(); };
	this.EngGetPageHeight = function() { return GetPageHeight(); };
	
	/*sfh
	******************************************************************************
	
		Function:		EngineInit
	
		Desc:			Initializes the engine.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function EngineInit(zoom, bAutoscroll)
	{
		m_metrics.zoom = zoom;
		m_metrics.zoomActual = 100;
		m_metrics.bAutoscroll = bAutoscroll;
	
		m_div = document.getElementById("divTileMap");
	}

	/*sfh
	******************************************************************************
	
		Function:		SetClientID
	
		Desc:			Called when the appshare server client id is set or 
						changed. We store the client id in a cookie so it is
						sent automatically with all image requests.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function SetClientID(cid)
	{
		// Save the client id in session cookie for tile downloads.
		document.cookie = "cid=" + cid + "; path=/";
	}
	
	/*sfh
	******************************************************************************
	
		Function:		SetSessType
	
		Desc:			Called when the appshare server sends the appshare type.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function SetSessType(sesstype)
	{
		if (sesstype != s_SESSION_TYPE_APPSHARE && sesstype != s_SESSION_TYPE_SLIDESHOW)
		{
			Debug.ConsoleOut("!!! ERROR !!! -- SetSessType -- invalid session type: " + sesstype);
		}
		else
		{
			m_session.sesstype = sesstype;
		}
	}
	
	/*sfh
	******************************************************************************
	
		Function:		BuildTileMap
	
		Desc:			Creates a grid of images based on the specified dimensions.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function BuildTileMap(numXTiles, numYTiles, tileWidth, tileHeight, screenWidth, screenHeight)
	{
		DestroyTileMap();
	
		m_metrics.screenWidth = screenWidth;
		m_metrics.screenHeight = screenHeight;
		
		m_metrics.numXTiles = numXTiles;
		m_metrics.numYTiles = numYTiles;
		m_metrics.tileWidth = tileWidth;
		m_metrics.tileHeight = tileHeight;
		
		m_tileMap = new Array(numXTiles * numYTiles + 1);
		
		// Size the container div.
		m_div.style.top = "0px";
		m_div.style.left = "0px";
		m_div.style.width = screenWidth + "px";
		m_div.style.height = screenHeight + "px";
		m_div.style.overflow = "hidden";

		m_metrics.offsetX = 0;			
		m_metrics.offsetY = 0;
		m_metrics.sharedRectWidth = screenWidth;	
		m_metrics.sharedRectHeight = screenHeight;		
	
		// Create the background tile (tile id 0). 
		AddTile(0, 0, 0, tileWidth, tileHeight);	
		
		var tileID = 1;  // Already made the background tile, so skip it.
		for (var yt = 0;  yt < numYTiles;  yt = yt + 1)
		{
			for (var xt = 0;  xt < numXTiles;  xt = xt + 1)
			{
				AddTile(tileID, xt * tileWidth, yt * tileHeight, tileWidth, tileHeight);
				tileID = tileID + 1;
			}
		}
		
		// Create the mouse cursor.
		m_cursor = CreateCursor();
		
		// Set the zoom and auto-scroll options.
		//
		// For slideshow sessions, set the startup zoom to auto-fit.
		// The server always sets the session type before the SetDimension() API.
		// The SetStartupZoom() function just sets the zoom in m_metrics, the Engine.EngBuildTileMap() call will apply the zoom.
		if (m_session.sesstype == s_SESSION_TYPE_SLIDESHOW)
		{
			m_metrics.zoom = "auto";
		}
		SetZoom(m_metrics.zoom);
		SetAutoScroll(m_metrics.bAutoscroll);
		
		// Add event handlers needed by the engine
		window.onresize = Engine.On_WindowResize;
	}
	
	/*sfh
	******************************************************************************
	
		Function:		DestroyTileMap
	
		Desc:			Destroys the grid by removing all the tiles and freeing all
						tile images.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function DestroyTileMap()
	{
		// Walk the tile map removing each tile, and ensuring each resource is
		// explicitly marked for garbage collection.
		if (m_tileMap)
		{
			for (var i = 0;  i < m_tileMap.length;  i = i + 1)
			{
				RemoveTile(i);
			}
		}
	}
	
	/*sfh
	******************************************************************************
	
		Function:		AddTile
	
		Desc:			Adds a tile to the tile map. 
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function AddTile(tileID, x, y, width, height)
	{
		// If a tile already exits with this tile id then remove it first.
		RemoveTile(tileID);
	
		var tile = new Object();
		tile.divtag = document.createElement("div");
		tile.imgtag = document.createElement("img");
		tile.imgtag2 = document.createElement("img"); 
		tile.download = new Object();
		tile.download.imgErrorRetries = 0;		// Current retry count
		tile.download.imgLoadTimerID = 0;		// Timer id for image download timeouts
		tile.download.batchID = 0;				// Batch id for url being downloaded
		tile.download.bLastInBatch = 0;			// True if the tile is the last one in the batch
		tile.download.fetchUrl = null;			// Url being downloaded
		tile.download.status = s_STATUS_NONE;	// Download status
		tile.download.fetchID = 0;				// All image downloads get a unqiue id - for debugging only 
		tile.refCache = new Object();			// Create image callback function references and cache them for repeated use later.
		tile.refCache.load = new Function("Engine.On_TileImageLoaded(" + tileID + ");");
		tile.refCache.error = new Function("Engine.On_TileImageError(" + tileID + ",'error');");
		tile.refCache.abort = new Function("Engine.On_TileImageError(" + tileID + ",'abort');");
	
		// Set up the container div
		tile.divtag.id = "divtile" + tileID;
		tile.divtag.style.position = "absolute";
		tile.divtag.style.left = x + "px";
		tile.divtag.style.top = y + "px";	
		tile.divtag.style.width = width + "px";
		tile.divtag.style.height = height + "px";
		tile.divtag.style.overflow = "hidden";
		tile.divtag.style.backgroundColor = "transparent";
		tile.divtag.style.border = "0px none";
		tile.divtag.style.outline = "none";
		tile.divtag.style.margin = "0px";
		tile.divtag.style.padding = "0px";
		
		// Set up image tag 1
		tile.imgtag.id = "tile" + tileID + "a";
		if (Debug.IsDebug()) { tile.imgtag.alt = tile.imgtag.id; }
		tile.imgtag.style.position = "absolute";
		tile.imgtag.style.left = "0px";
		tile.imgtag.style.top = "0px";	
		tile.imgtag.style.visibility = "hidden";	
		tile.imgtag.style.backgroundColor = "transparent";
		tile.imgtag.style.border = "0px none";
		tile.imgtag.style.outline = "none";
		tile.imgtag.style.margin = "0px";
		tile.imgtag.style.padding = "0px";
		tile.imgtag.style.msInterpolationMode = "bicubic"; 
		
		// Set up image tag 2 - two image tags are used for double buffered rendering
		tile.imgtag2.id = "tile" + tileID + "b";
		if (Debug.IsDebug()) { tile.imgtag.alt = tile.imgtag.id; }
		tile.imgtag2.style.position = "absolute";
		tile.imgtag2.style.left = "0px";
		tile.imgtag2.style.top = "0px";	
		tile.imgtag2.style.visibility = "hidden";
		tile.imgtag2.style.backgroundColor = "transparent";
		tile.imgtag2.style.border = "0px none";
		tile.imgtag2.style.outline = "none";
		tile.imgtag2.style.margin = "0px";
		tile.imgtag2.style.padding = "0px";
		tile.imgtag2.style.msInterpolationMode = "bicubic"; 
	
		// Add it to the map.
		m_tileMap[tileID] = tile;
	
		// Tile id 0 is the background tile so don't add it to the grid.
		if (tileID !== 0)
		{
			tile.divtag.appendChild(tile.imgtag);
			tile.divtag.appendChild(tile.imgtag2);
			m_div.appendChild(tile.divtag);
		}	
	}
	
	/*sfh
	******************************************************************************
	
		Function:		RemoveTile
	
		Desc:			Removes the tile, and ensures the resources are
						explicitly marked for garbage collection.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function RemoveTile(tileID)
	{
		var tile = GetTile(tileID);
		if (tile)
		{
			m_div.removeChild(tile.imgtag);
			tile.imgtag = null;
			tile.img = null;
			m_tileMap[tileID] = null;
		}
	}
	
	/*sfh
	******************************************************************************
	
		Function:		SetZoom
	
		Desc:			Sets the zoom (magnification) to the specified 
						percentage. 
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function SetZoom(zoom)
	{
		if (!IsZoomSupported())
		{
			zoom = 100;
		}
		m_metrics.zoom = zoom;
		m_metrics.zoomActual = zoom;
		ApplyZoom();
	}
	
	/*sfh
	******************************************************************************
	
		Function:		ApplyZoom
	
		Desc:			Applies the current zoom setting to the tile div. If 
						Auto-Fit is enabled then the zoom factor is recalulated. 
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function ApplyZoom()
	{
		var iNewZoom = 100;
	
		// If 'auto' or 'width', then calculate the actual zoom needed.
		if (m_metrics.zoom == "auto" || m_metrics.zoom == "width")
		{
			var iWindowWidth = GetPageWidth();
			var iWindowHeight = (m_metrics.zoom == "width") ? 500000 : GetPageHeight();
			var iShareWidth = m_metrics.sharedRectWidth;
			var iShareHeight = m_metrics.sharedRectHeight;
		
			if (!isNaN(iShareWidth) && !isNaN(iShareHeight))
			{
				iNewZoom = CalcBestFitZoom(iWindowWidth, iWindowHeight, iShareWidth, iShareHeight);
				
				// Don't scale up when auto zooming
				// CA - the scale up limit is set to 200, and the acale down limit is 25
				iNewZoom = Math.max(25, Math.min(iNewZoom, 200));
			}
		}
		else
		{
			iNewZoom = parseInt(m_metrics.zoom);
		}
		if (isNaN(iNewZoom))
		{
			Debug.ConsoleOut("WARNING --- invalid zoom specified: " + iNewZoom);
			iNewZoom = "100";
		}
		
		// Force zoom into bounds of 25% to 200%
		iNewZoom = Math.max(25, Math.min(200, iNewZoom));
		m_metrics.zoomActual = iNewZoom;

		// Apply this zoom to the tile div, and calc new div offset and mouse position.
		m_div.style.zoom = iNewZoom + "%";
		
		var zoomPercent = iNewZoom / 100;
		m_div.style.left = Math.round((0 - m_metrics.offsetX) * zoomPercent) + "px";
		m_div.style.top = Math.round((0 - m_metrics.offsetY) * zoomPercent) + "px";
		m_div.style.width = (m_metrics.sharedRectWidth + m_metrics.offsetX) +  "px";
		m_div.style.height = (m_metrics.sharedRectHeight + m_metrics.offsetY) + "px";

		m_cursor.div.style.left = Math.round((m_metrics.cursorPosX - m_metrics.offsetX) * zoomPercent) + "px";
		m_cursor.div.style.top = Math.round((m_metrics.cursorPosY - m_metrics.offsetY) * zoomPercent) + "px";
	}
	
	/*sfh
	******************************************************************************
	
		Function:		CalcBestFitZoom
	
		Desc:			Calculates the zoom factor representing the best fit 
						based on the size of the shared area relative to the
						size of this browser window's viewable area.
	
		Returns:		int
	
	******************************************************************************
	efh*/
	
	function CalcBestFitZoom(iWindowWidth, iWindowHeight, iShareWidth, iShareHeight)
	{
		if (!IsZoomSupported()) { return "100"; }

		var iNewZoom = 100;
	
		try
		{
			var iZoomX = (iWindowWidth / iShareWidth) * 100;
			var iZoomY = (iWindowHeight / iShareHeight) * 100;
	
			iNewZoom = Math.round(Math.min(iZoomX, iZoomY) - s_POINT_FIVE);
		}
		catch(e)
		{
			Debug.ConsoleOut("CalcBestFit - exception thrown: " + e.description);
			iNewZoom = 100;		
		} 
		return iNewZoom;
	}

	/*sfh
	******************************************************************************
	
		Function:		On_WindowResize
	
		Desc:			Called when the browser window is resized. We need to
						re-calculate the zoom level if auto zoom is enabled.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.On_WindowResize = function(evt)
	{
		ApplyZoom();	
	};
	
	/*sfh
	******************************************************************************
	
		Function:		AutoScroll
	
		Desc:			Enables or disables the auto-scroll feature, meaning the
						screen will scroll automatically such that the moderator's
						mouse cursor is always centered. 
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function SetAutoScroll(bAutoScroll)
	{
		// TODO
	}
	
	/*sfh
	******************************************************************************
	
		Function:		On_TileImageLoaded
	
		Desc:			Callback called when a tile's image tag finishes loading 
						an image.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.On_TileImageLoaded = function(tileID)
	{
		var tile = GetTile(tileID);

		// Check if this is simply the tiny blank.gif we load into the background buffer
		// to release image memory - if so then ignore.
		if (tile.imgtag2.src.indexOf("blank.gif") != -1 || tile.imgtag2.src.indexOf("blank-debug.gif") != -1)
		{
			Debug.ConsoleOut("!!!!!!!!!!!!!!!!! SHORT CUT!  On_TileImageLoaded -- tid=" + tileID + " --- src=" + tile.imgtag2.src);
			return;		
		}

		// Cancel the timeout on this tile's image download.
		if (tile.download.imgLoadTimerID)
		{
			window.clearTimeout(tile.download.imgLoadTimerID);
			tile.download.imgLoadTimerID = 0;
		}
	
		// Remove callbacks for the tile's buffer image tag so we don't get redundant calls (FireFox) and
		// so this handles doesn't get called for loading of "blank.gif".
		tile.imgtag2.onload = null;
		tile.imgtag2.onerror = null;
		tile.imgtag2.onabort = null;
		
		tile.download.status = s_STATUS_DONE;
		
		// If this download has the same batch id as the current batch then
		// we can delay rendering by simply adding it to the current batch.
		if (tile.download.batchID == m_batch.id && tile.download.batchID !== 0)
		{
			Debug.ConsoleOut("TILE IMAGE LOADED --- tile=" + tileID + "; batchid=" + tile.download.batchID + " ;  --- Adding to batch. --- bLastInBatch=" + tile.download.bLastInBatch + "--- url=" + tile.imgtag2.src);
			BatchAdd(tileID);
			
			// If this is the last tile in the batch, then we can render the batch now.
			if (tile.download.bLastInBatch) { BatchRender(); }
		}
		else
		{
			// Render the tile right away.
			Debug.ConsoleOut("TILE IMAGE LOADED --- tile=" + tileID + "; batchid=" + tile.download.batchID + " ;  --- Rendering now. ---  bLastInBatch=" + tile.download.bLastInBatch + "--- url=" + tile.imgtag2.src);
			RenderTile(tileID);
		}
	};
	
	/*sfh
	******************************************************************************
	
		Function:		On_TileImageError
	
		Desc:			Callback called when a tile's image tag encounters an
						error or times out while loading an image.

						"cause" can be one of: error, abort, timeout
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.On_TileImageError = function(tileID, cause)
	{
		var tile = GetTile(tileID);

		// Check if this is simply the tiny blank.gif we load into the background buffer
		// to release image memory - if so then ignore.
		if (tile.imgtag2.src.indexOf("blank.gif") != -1 || tile.imgtag2.src.indexOf("blank-debug.gif") != -1)
		{
			return;		
		}

		// Cancel the timeout on this tile's image download.
		if (tile.download.imgLoadTimerID)
		{
			window.clearTimeout(tile.download.imgLoadTimerID);
			tile.download.imgLoadTimerID = 0;
		}

		// Remove callbacks for the tile's buffer image tag so we don't get redundant calls (FireFox) and
		// so this handler doesn't get called for loading of "blank.gif".
		tile.imgtag2.onload = null;
		tile.imgtag2.onerror = null;
		tile.imgtag2.onabort = null;

		// Check if the maximum number of retries has been exceeded.
		if (tile !== null && tile.download.imgErrorRetries < s_IMAGE_LOAD_MAX_RETRIES)
		{ 
			// Re-Try the image download in a few seconds.
			tile.download.imgErrorRetries = tile.download.imgErrorRetries + 1;
			tile.imgLoadTimeout = window.setTimeout("Engine.FetchTile(" + tileID + "," + tile.download.batchID + ",'" + tile.download.fetchUrl + "'," + tile.download.bLastInBatch + ");", 1000);

			Debug.ConsoleOut("TILE IMAGE " + cause + " DOWNLOADING --- tile=" + tileID + "  --- retry # " + tile.download.imgErrorRetries + " in 2 secs");
			Debug.ConsoleOutProps(tile.download);
		}
		else if (tile === null)
		{
			Debug.ConsoleOut("TILE IMAGE " + cause + " DOWNLOADING --- tile=" + tileID + "  --- INVALID TILE ID.");	
		}
		else
		{
			Debug.ConsoleOut("TILE IMAGE " + cause + " DOWNLOADING --- tile=" + tileID + "  --- MAX RETRIES EXCEEDED.");	
			Debug.ConsoleOutProps(tile.download);
		}
	};

	/*sfh
	******************************************************************************
	
		Function:		MarkTileDirty
	
		Desc:			Forces the specified tile to re-download its image.
						If the tile id is negative, then set it to the background 
						tile.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function MarkTileDirty(iTileID, iBatchID, url, bLastInBatch)
	{
		// If no iBatchID is given then use 0.
		if (!iBatchID)
		{
			iBatchID = 0;		
		} 

		// If the tile id is negative then we must use the background tile image.
		if (iTileID < 0)
		{
			// The dirty tile id was negative, so use the
			// background tile image.
			var tileBK = GetTile(0);
			url = tileBK.imgtag2.src;
			iTileID = Math.abs(iTileID);
		}

		// Sanity check.
		if (!url || url.length < 1)
		{
			Debug.ConsoleOut("!!!!!!!!!!! MarkTileDirty -- url is BLANK!!  url=" + url);
			return;
		}
		Engine.FetchTile(iTileID, iBatchID, url, bLastInBatch);
	}
	
	/*sfh
	******************************************************************************
	
		Function:		FetchTile
	
		Desc:			Loads a new image into a tile's img object.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.FetchTile = function(tileID, iBatchID, url, bLastInBatch)
	{
		var tile = GetTile(tileID);

		// If a previous download is in progress for this tile then kill the timeout timer.
		if (tile.download.imgLoadTimerID)
		{
			window.clearTimeout(tile.download.imgLoadTimerID);
			tile.download.imgLoadTimerID = 0;
		}
	
		// If the batch id has changed then render the current batch and 
		// start a new batch.
		if (m_batch.id != iBatchID)
		{
			BatchStart(iBatchID);  // Renders the current and starts a new batch.
		}
	
		// Provide unique number for each fetch request
		if (!this.FetchTile.iFetchCounter) { this.FetchTile.iFetchCounter = 0; }
		this.FetchTile.iFetchCounter = this.FetchTile.iFetchCounter + 1;

		// Add callbacks so we get notified when load is complete.
		tile.imgtag2.onload = tile.refCache.load;
		tile.imgtag2.onerror = tile.refCache.error;
		tile.imgtag2.onabort = tile.refCache.abort;
		
		// Create a timeout timer so we can retry if the image loading takes too long.
		tile.download.imgLoadTimerID = window.setTimeout("Engine.On_TileImageError(" + tileID + ",'timeout');", s_IMAGE_LOAD_TIMEOUT);

		// Update download stats.
		tile.download.imgErrorRetries = 0;
		tile.download.batchID = iBatchID;
		tile.download.bLastInBatch = bLastInBatch;
		tile.download.fetchUrl = url;
		tile.download.status = s_STATUS_DOWNLOADING;
		tile.download.fetchID = this.FetchTile.iFetchCounter; 

		// Start the image load.
		tile.imgtag2.src = url;

		// Special case: when updating the background tile (tileID 0),
		// we also must set all tiles to the BK tile image.
		if (tileID === 0)
		{
			for (var i = 1;  i < m_tileMap.length;  i = i + 1)
			{
				this.FetchTile(i, iBatchID, url);
			}
		}
	};

	/*sfh
	******************************************************************************
	
		Function:		BatchStart
	
		Desc:			Starts a new batch update. Tile images will be downloaded
						but rendering will be delayed until the batch is complete
						(or times out).
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function BatchStart(iBatchID)
	{
		// If there is already a batch in progress then render it.
		BatchRender();

		Debug.ConsoleOut("BatchStart -- starting a new batch; batchid=" + iBatchID);
		
		// Set up this batch.
		m_batch.id = iBatchID;
		m_batch.timer = 0;   // Start the timer when the first image is added.
		m_batch.queue = new Array();
	}

	/*sfh
	******************************************************************************
	
		Function:		BatchAdd
	
		Desc:			Adds a tile id to the current batch.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function BatchAdd(tileID)
	{
		// If this is the first item in the batch then start the timeout timer.
		if (!m_batch.timer)
		{
			m_batch.timer = window.setTimeout("Engine.BatchTimeout();", s_BATCH_RENDER_TIMEOUT);
		}

		// Add the tile id to the end of the queue.
		m_batch.queue.push(tileID);
	}

	/*sfh
	******************************************************************************
	
		Function:		BatchTimeout
	
		Desc:			Called when the batch delay has expired and the batch
						must be rendered now.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.BatchTimeout = function ()
	{
		Debug.ConsoleOut("BatchTimeout -- tile count=" + m_batch.queue.length + "; batchid=" + m_batch.id);
	
		m_batch.timer = 0;
	
		// Render the batch so far, then start a new batch with the same id so
		// remaining tiles in this batch are rendered together.
		BatchStart(m_batch.id);
	};

	/*sfh
	******************************************************************************
	
		Function:		BatchRender
	
		Desc:			Renders all tiles that have a new image downloaded within
						the current batch update.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function BatchRender()
	{
		// Cancel the timer.
		if (m_batch.timer)
		{
			window.clearTimeout(m_batch.timer);
			m_batch.timer = 0;
		}

		Debug.ConsoleOut("BatchRender -- tile count=" + m_batch.queue.length + " ; batchid=" + m_batch.id);
	
		// Render any tiles in the batch queue.
		for (var i = 0;  i < m_batch.queue.length;  i = i + 1)
		{
			RenderTile(m_batch.queue[i]);		
		}		
		
		// Clean up for next batch.
		m_batch.id = 0;
		m_batch.queue.length = 0;
	}

	/*sfh
	******************************************************************************
	
		Function:		RenderTile
	
		Desc:			Renders tile, meaning that any image that has been 
						downloaded for this tile but not yet displayed will
						be displayed.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function RenderTile(tileID)
	{
		var tile = GetTile(tileID);
		if (!tile) { Debug.ConsoleOut("RenderTile -- invalid tile id !!!!"); return; }
		
		// Check if there is an image downloaded.
		if (tile.download.status == s_STATUS_DONE)
		{
			Debug.ConsoleOut("RenderTile -- tid=" + tileID);
			
			// Don't try to display the background tile.
			if (tileID !== 0)
			{
				// Put the buffered image tag topmost in z-order
				tile.imgtag2.style.zIndex = "2000";  // Higher zIndex means topmost on screen.
				tile.imgtag.style.zIndex = "1000";
				
				// Show it
				tile.imgtag2.style.visibility = "visible"; 
				tile.imgtag.style.visibility = "hidden";

				// Swap buffered image reference for screen image reference.
				var tmp = tile.imgtag;
				tile.imgtag = tile.imgtag2;
				tile.imgtag2 = tmp;
	
				// Clear the old image to release memory - especially important for IE6 and 
				// archiving playback which has huge scene images.  
				//tile.imgtag2.src = Debug.IsDebug() ? "blank-debug.gif" : "blank.gif";
			}

			// Clear download stats.
			tile.download.imgErrorRetries = 0;
			tile.download.batchID = 0;
			tile.download.fetchUrl = null;
			tile.download.status = s_STATUS_NONE;
		}
		else
		{
			Debug.ConsoleOut("!!!! RenderTile called but no image downloaded for tile.  tid=" + tileID + "; status=" + tile.download.status + "; src=" + tile.imgtag2.src);		
		}
	}
	
	/*sfh
	******************************************************************************
	
		Function:		SetTileCropped
	
		Desc:			Called during archive playback to set the specified tile 
						to a cropped section of the source PNG.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function SetTileCropped(iTileID, urlPNG, pngX, pngY)
	{
		var tile = GetTile(iTileID);
		if (tile)
		{
			var bOpera = IsOpera();
	
			// Set up the callbacks.
			if (!tile.bArchivePlayerInit)
			{
				tile.bArchivePlayerInit = true;
				if (!bOpera)
				{
					tile.imgtag.onload = new Function("Engine.On_TAG_ARCHIVE_TileImageLoaded(" + iTileID + ");");
					tile.imgtag2.onload = new Function("Engine.On_TAG_ARCHIVE_TileImageLoaded(" + iTileID + ");");
				}
				tile.imgtag.style.visibility = "visible";
				tile.imgtag2.style.visibility = "visible";
			}
			
			// Check if this image is already loaded into this tile.
			if (GetFilenameFromURL(urlPNG) != GetFilenameFromURL(tile.imgtag.src))
			{
				// Set image positon		
				tile.imgtag2.style.left = (-1 * pngX) + "px";
				tile.imgtag2.style.top  = (-1 * pngY) + "px";	
	
				// Load the new image into the hidden IMG tag and set position.
				tile.imgtag2.bBlank = false;
				tile.imgtag2.src = urlPNG;
	
				// Opera simply refuses to generate an onload event for images 
				// that have already been loaded by some other element, therefore for 
				// Opera users we'll have to live with some flashing between scenes.
				if (bOpera)
				{
					window.setTimeout("Engine.On_TAG_ARCHIVE_TileImageLoaded(" + iTileID + ");", 50);
				}
			}
			else
			{
				// Just reposition the existing image.
				tile.imgtag.style.left = (-1 * pngX) + "px";
				tile.imgtag.style.top  = (-1 * pngY) + "px";	
			}
		}
	}
	
	/*sfh
	******************************************************************************
	
		Function:		On_TAG_ARCHIVE_TileImageLoaded
	
		Desc:			Called during archive playback when an image is finished
						loading into a tile's img tag.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.On_TAG_ARCHIVE_TileImageLoaded = function(iTileID)
	{
		var tile = GetTile(iTileID);
		if (tile && !tile.imgtag2.bBlank)
		{
			// Now swap IMG tags for flicker free draw.
			tile.imgtag2.style.zIndex = "2000";
			tile.imgtag.style.zIndex = "1000";
	
			var tmp = tile.imgtag;
			tile.imgtag = tile.imgtag2;
			tile.imgtag2 = tmp;
	
			// Free the second image 
			// (for IE6 only - which creates in-memory bitmap for every instance of same image!) 
			tile.imgtag2.bBlank = true;
			tile.imgtag2.src = "blank.gif";
		}
	};
	
	/*sfh
	******************************************************************************
	
		Function:		GetTile
	
		Desc:			Returns a tile object for the specified tile id,
						or null.
	
		Returns:		tile
	
	******************************************************************************
	efh*/
	
	function GetTile(tileID)
	{
		return (m_tileMap) ? m_tileMap[tileID] : null;
	}
	
	/*sfh
	******************************************************************************
	
		Function:		CreateCursor
	
		Desc:			Creates a new Cursor object and intializes it.
	
		Returns:		Object
	
	******************************************************************************
	efh*/
	
	function CreateCursor()
	{
		var cursor = new Object();
		
		// Create the container div & sub-div
		cursor.div = document.createElement("div");
		cursor.div.style.position = "absolute";
		cursor.div.style.left = "0px";
		cursor.div.style.top = "0px";
		cursor.div.style.zIndex = "" + (m_metrics.numXTiles * m_metrics.numYTiles) + 10000;
		cursor.div.style.zoom = "100%";
	
		cursor.divCursor = document.createElement("div");
		cursor.divCursor.style.position = "static";
		cursor.divCursor.style.left = "0px";
		cursor.divCursor.style.top = "0px";
	
		// Create the img tag and image object properties	
		cursor.imgtag = document.createElement("img");
		cursor.imgtag.style.position = "absolute";
		cursor.imgtag.style.left = "0px";
		cursor.imgtag.style.top = "0px";
		
		cursor.img = null;
		cursor.imgLoadTimeout = 0;
	
		// Create the tooltip
		cursor.tooltip = document.createElement("div");
		cursor.tooltip.style.position = "static";
		cursor.tooltip.style.backgroundColor = "#FFFFE1"; // use the same color as the other appsharing clients
		cursor.tooltip.style.border = "1px solid black";
		cursor.tooltip.style.color = "black";
		cursor.tooltip.style.font = "normal normal normal 7pt sans-serif";
	    cursor.tooltip.style.marginTop = "34px";    // location below the cursor image
		cursor.tooltip.style.paddingTop = "0px";
		cursor.tooltip.style.paddingBottom = "0px";
		cursor.tooltip.style.paddingLeft = "2px";
		cursor.tooltip.style.paddingRight = "2px";	
		//cursor.tooltip.style.opacity = "0.75";  // CSS2 only!
		cursor.tooltip.style.visibility = "hidden";
	
		// Build the object hierarchy, adding the cursor's containing div and tooltip div to the main div.
		cursor.divCursor.appendChild(cursor.imgtag);
		cursor.div.appendChild(cursor.divCursor);
		cursor.div.appendChild(cursor.tooltip);
		document.body.appendChild(cursor.div);
	
		// Register the callbacks for image load events.
		cursor.imgtag.onload = new Function("Engine.On_TAG_CursorImageLoaded();");	
		cursor.imgtag.onerror = new Function("Engine.On_TAG_CursorImageError();");	
		cursor.imgtag.onabort = new Function("Engine.On_TAG_CursorImageAbort();");	
	
		return cursor;
	}
	
	/*sfh
	******************************************************************************
	
		Function:		MarkCursorImageDirty
	
		Desc:			Triggers the download of a new mouse cursor image. 
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.MarkCursorImageDirty = function(url)
	{
		// Since these image event callbacks don't require params, we can create
		// and cache the function refs.
		if (!this.MarkCursorImageDirty.funcLoad)
		{
			this.MarkCursorImageDirty.funcLoad = new Function("Engine.On_OBJECT_CursorImageLoaded();");	
			this.MarkCursorImageDirty.funcError = new Function("Engine.On_OBJECT_CursorImageError();");	
			this.MarkCursorImageDirty.funcAbort = new Function("Engine.On_OBJECT_CursorImageAbort();");			
		}
	
		// If another download is in progress then kill the timeout timer.
		if (m_cursor.imgLoadTimeout)
		{
			window.clearTimeout(m_cursor.imgLoadTimeout);
			m_cursor.imgLoadTimeout = 0;
		}
	
		// Download new cursor image.
		m_cursor.img = CreateImage();
		m_cursor.img.onload = this.MarkCursorImageDirty.funcLoad;
		m_cursor.img.onerror = this.MarkCursorImageDirty.funcError; 
		m_cursor.img.onabort = this.MarkCursorImageDirty.funcAbort;
		m_cursor.imgLoadTimeout = window.setTimeout("Engine.On_OBJECT_CursorImageError();", s_IMAGE_LOAD_TIMEOUT);
		m_cursor.img.src = url;
	};
	
	/*sfh
	******************************************************************************
	
		Function:		MoveCursor
	
		Desc:			Sets a new position for the cursor. 
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function MoveCursor(x,y)
	{
		m_metrics.cursorPosX = x;
		m_metrics.cursorPosY = y;

		ApplyZoom();
		
		// Show the tooltip again for 5 seconds if it is active.
		if (IsTooltipActive())
		{
			UpdateTooltipVisibility(true);
		}
	}

	/*sfh
	******************************************************************************
	
		Function:		SetCursorCropped
	
		Desc:			Sets the cursor image. This method supports multi-cursor 
						image strips where the image strip is cropped based on 
						the params.
						
						Called during archive playback.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function SetCursorCropped(urlImg, imgX, imgY, cursorWidth, cursorHeight)
	{
		if (!SetCursorCropped.bInit || cursorWidth != SetCursorCropped.iCursorWidth || cursorHeight != SetCursorCropped.iCursorHeight)
		{
			// Set up the cursor object for handling cropping within a multi-image source.
			// This is only done during archive playback.
			m_cursor.divCursor.style.overflow = "hidden";
			m_cursor.divCursor.style.width = cursorWidth + "px";
			m_cursor.divCursor.style.height = cursorHeight + "px";
			
			SetCursorCropped.iCursorWidth = cursorWidth;
			SetCursorCropped.iCursorHeight = cursorHeight;
			SetCursorCropped.bInit = true;
		}
	
		var str = "rect(" + imgY + "px " + (imgX + cursorWidth) + "px " + (imgY + cursorHeight) + "px " + imgX + "px)";
		m_cursor.imgtag.style.clip = str;
	
		m_cursor.imgtag.src = urlImg;
		m_cursor.imgtag.style.left = (-1 * imgX) + "px";
		m_cursor.imgtag.style.top = (-1 * imgY) + "px";
	}
	
	/*sfh
	******************************************************************************
	
		Function:		On_OBJECT_CursorImageLoaded
	
		Desc:			Callback called when the cursor's Image object finishes 
						loading an image.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.On_OBJECT_CursorImageLoaded = function()	
	{
		// Cancel the image load timeout.
		if (m_cursor.imgLoadTimeout)
		{
			window.clearTimeout(m_cursor.imgLoadTimeout);
			m_cursor.imgLoadTimeout = 0;
		}
		
		if (m_cursor.img)
		{
			// Resize the img tag base on cursor image size.
			m_cursor.imgtag.style.width = m_cursor.img.width + "px";
			m_cursor.imgtag.style.height = m_cursor.img.height + "px";	
			
			// Give the image to the IMG tag.
			m_cursor.imgtag.src = m_cursor.img.src;
			
			// Safe to let the image object get garbage collected.
			m_cursor.img = null;
		}
	};
	
	/*sfh
	******************************************************************************
	
		Function:		On_OBJECT_CursorImageError
	
		Desc:			Callback called when the cursor's Image object encounters 
						an error while downloading an image.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.On_OBJECT_CursorImageError = function()	
	{
		// Retry the cursor image download after a few seconds.
		m_cursor.imgLoadTimeout = window.setTimeout("Engine.MarkCursorImageDirty(null,null);", 2000);	 
	
		Debug.ConsoleOut("IMAGE OBJECT ERROR DOWNLOADING cursor.  Retrying in 2 secs.");
	};
	
	/*sfh
	******************************************************************************
	
		Function:		On_OBJECT_CursorImageAbort
	
		Desc:			Callback called when the cursor's Image object has a 
						downloaded aborted.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.On_OBJECT_CursorImageAbort = function()	
	{
		Debug.ConsoleOut("IMAGE OBJECT ABORT DOWNLOADING cursor.");
	};
	
	/*sfh
	******************************************************************************
	
		Function:		On_TAG_CursorImageLoaded
	
		Desc:			Callback called when the cursor's image TAG finishes 
						loading an image. 
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.On_TAG_CursorImageLoaded = function()
	{
		// Show the cursor now that an image is loaded.
		m_cursor.imgtag.style.visibility = "visible";
		
		// We only need a callback the first time for tags.
		m_cursor.imgtag.onload = "";
	};
	
	/*sfh
	******************************************************************************
	
		Function:		On_TAG_CursorImageError
	
		Desc:			Callback called when the cursor's image TAG encounters
						an error while loading an image. 
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.On_TAG_CursorImageError = function()
	{
		// Since an image object already downloaded the image, this
		// should never happen, except perhaps if the user's browser
		// cache is turned off.
		Debug.ConsoleOut("IMAGE TAG ERROR DOWNLOADING cursor.");
	};
	
	/*sfh
	******************************************************************************
	
		Function:		On_TAG_CursorImageAbort
	
		Desc:			Callback called when the cursor's image TAG has a abort
						event thrown. Since the image was already downloaded in
						an image object this should never happen.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.On_TAG_CursorImageAbort = function()
	{
		// Since an image object already downloaded the image, this
		// should never happen, except perhaps if the user's browser
		// cache is turned off.
		Debug.ConsoleOut("IMAGE TAG ABORT DOWNLOADING cursor.");
	};
	
	/*sfh
	******************************************************************************
	
		Function:		NotifySharing
	
		Desc:			Called when switching between the states where nothing is
						shared or something is shared.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function NotifySharing(bSharing) 
	{
		m_session.bSharing = bSharing;
	}
	
	/*sfh
	******************************************************************************
	
		Function:		ShowTooltip
	
		Desc:			Shows or hides the tooltip, or set new text.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function ShowTooltip(bShow, txt) 
	{
	    // Don't show the tooltip if nothing is shared
	    if(!m_session.bSharing) { return; }
	    
		if (!bShow || !txt)
		{
			txt = "";	
		}
		
		// The tooltip text cannot be longer than s_TOOLTIP_LINE_MAX_CHARS characters - '...' are added at the end of the line
		// Alternatively, two lines are displayed (break at first space), each line size reduced to maximum s_TOOLTIP_LINE_MAX_CHARS chars
		var strTxt = new String(txt);
		var strTxt1,strTxt2,strTxtToShow;
		// break at first ' '
		var nPos = strTxt.indexOf(" ");
		if( nPos == -1)
		{
		    if( strTxt.length <= s_TOOLTIP_LINE_MAX_CHARS)
		    {
			    strTxtToShow = strTxt;
			}
			else
			{
			    strTxtToShow = strTxt.substring(0,s_TOOLTIP_LINE_MAX_CHARS-3);
			    strTxtToShow = strTxtToShow.concat("...");
			}
		}
		else
		{
		    // break into two text lines
		    strTxt1 = strTxt.substring(0,nPos);
		    strTxt2 = strTxt.substring(nPos+1,strTxt.length);
		    if( strTxt1.length > s_TOOLTIP_LINE_MAX_CHARS)
		    {
		        strTxt1 = strTxt1.substring(0,s_TOOLTIP_LINE_MAX_CHARS-3);
		        strTxt1 = strTxt1.concat("...");
		    }
		    if( strTxt2.length > s_TOOLTIP_LINE_MAX_CHARS)
		    {
		        strTxt2 = strTxt2.substring(0,s_TOOLTIP_LINE_MAX_CHARS-3);
		        strTxt2 = strTxt2.concat("...");
		    }
		    strTxtToShow = strTxt1;
		    strTxtToShow = strTxtToShow.concat("<br />");   
		    strTxtToShow = strTxtToShow.concat(strTxt2);
		}
		
		m_cursor.tooltip.tipActivated = bShow;
		m_cursor.tooltip.innerHTML = strTxtToShow;
		UpdateTooltipVisibility(bShow);
	}
	
	/*sfh
	******************************************************************************
	
		Function:		SetLocalCursor
	
		Desc:			Sets the local mouse cursor to specified shape. 
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function SetLocalCursor(cursor)
	{
		if (!cursor)
		{
			cursor = "default";
		}
		m_div.style.cursor = cursor;
	}
	
	/*sfh
	******************************************************************************
	
		Function:		UpdateTooltipVisibility
	
		Desc:			Shows or hides the tooltip. When showing the tooltip a 
						timer is created that hides the tooltip after 5 seconds of
						no mouse moves. When the mouse moves again the tooltip
						is then displayed again.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function UpdateTooltipVisibility(bShow) 
	{
		// Kill the timeout
		if (m_cursor.tooltip.tipTimeout)
		{
			window.clearTimeout(m_cursor.tooltip.tipTimeout);
			m_cursor.tooltip.tipTimeout = 0;
		}
		
		if (bShow)
		{
			m_cursor.tooltip.style.visibility = "visible";
			// Set a timeout to hide the cursor if the active presenter's mouse 
			// doesn't move for 5 seconds. Mousemoves will trigger a call to 
			// UpdateTooltipVisibility with bShow=true thus restarting the timer 
			// on each mouse move.
			m_cursor.tooltip.tipTimeout = window.setTimeout("Engine.OnHideTooltip();", 5000);
		}
		else
		{
			m_cursor.tooltip.style.visibility = "hidden";
		}
	}

	/*sfh
	******************************************************************************
	
		Function:		OnHideTooltip
	
		Desc:			Called when the tooltip needs to be hidden because the
						active presenter's mouse has been still for 5 seconds.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.OnHideTooltip = function() 
	{
		UpdateTooltipVisibility(false);
	};
	
	/*sfh
	******************************************************************************
	
		Function:		IsTooltipActive
	
		Desc:			Returns true if the tooltip is active. The tooltip may be
						hidden since it disappears after 5 seconds of no mouse
						moves, but the tooltip is active if it will reappear
						when the mouse moves again.
	
		Returns:		boolean
	
	******************************************************************************
	efh*/
	
	function IsTooltipActive()
	{
		return (m_cursor.tooltip.tipActivated) ? true : false;
	}
	
	/*sfh
	******************************************************************************
	
		Function:		SetScreenOffset
	
		Desc:			Moves the top-left corner of the containing div based on 
						the specified offset.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function SetScreenOffset(x,y) 
	{  
		m_metrics.offsetX = Math.max(x, 0);
		m_metrics.offsetY = Math.max(y, 0);

		ApplyZoom();
	
		Debug.ConsoleOut("Screen Offset changed to (" + (0-x) + "," + (0-y) + ")");
		Debug.ConsoleOut("WARNING:  EngSetScreenOffset deprecated");
	}
	
	/*sfh
	******************************************************************************
	
		Function:		SetSharedRect
	
		Desc:			Sets the bounding rect of the display area.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	function SetSharedRect(left,top,width,height) 
	{  
		// Ensure the offset is always positive, meaning the div can only be pushed
		// towards the upper-left.
		if (left < 0) { width = width - left;  left = 0; }
		if (top < 0) { height = height - top;  top = 0; }
		
		m_metrics.offsetX = left;
		m_metrics.offsetY = top;

		// Crop the width and height to ensure it does not overflow the moderator's 
		// screen rect.
		if (left + width > m_metrics.screenWidth)  { width = Math.max(m_metrics.screenWidth - left, 1); }  
		if (top + height > m_metrics.screenHeight)  { height = Math.max(m_metrics.screenHeight - top, 1); }

		m_metrics.sharedRectWidth = width;
		m_metrics.sharedRectHeight = height;

		ApplyZoom();

		Debug.ConsoleOut("Shared rect changed to (l=" + (0-left) + ",t=" + (0-top) + ",w=" + width + ",h=" + height + ")");
	}
	
	/*sfh
	******************************************************************************
	
		Function:		IsOpera
	
		Desc:			Returns true if Opera is the current browser. 
	
		Returns:		boolean
	
	******************************************************************************
	efh*/
	
	function IsOpera()
	{
		if (!IsOpera.bChecked)
		{
			var ua = navigator.userAgent.toLowerCase();
			IsOpera.bIsOpera = (ua.indexOf("opera") != -1) ? true : false;
			IsOpera.bChecked = true;
		}
		return IsOpera.bIsOpera;
	}
	
	/*sfh
	******************************************************************************
	
		Function:		GetPageWidth
	
		Desc:			Returns the page width, meaning the visible width of the
						Aegis window or frame.
	
		Returns:		int
	
	******************************************************************************
	efh*/
	
	function GetPageWidth() 
	{
		var iWidth = 0;
		var de = document.documentElement;
		if(typeof(window.innerWidth) == 'number' )
		{
			//Non-IE
			iWidth = window.innerWidth;
		} 
		else if( de && ( de.clientWidth || de.clientHeight ) ) 
		{
			//IE 6+ in 'standards compliant mode'
			iWidth = de.clientWidth;
		} 
		else if( document.body && ( document.body.clientWidth || document.body.clientHeight ) )
		{
	    	//IE 4 compatible
	    	iWidth = document.body.clientWidth;
		}
		return iWidth;
	}
	
	/*sfh
	******************************************************************************
	
		Function:		GetPageHeight
	
		Desc:			Returns the page height, meaning the visible height of the
						Aegis window or frame.
	
		Returns:		int
	
	******************************************************************************
	efh*/
	
	function GetPageHeight() 
	{
		var iHeight = 0;
		var de = document.documentElement;
		if(typeof(window.innerHeight) == 'number' )
		{
			//Non-IE
			iHeight = window.innerHeight;
		} 
		else if( de && ( de.clientWidth || de.clientHeight ) ) 
		{
			//IE 6+ in 'standards compliant mode'
			iHeight = de.clientHeight;
		} 
		else if( document.body && ( document.body.clientWidth || document.body.clientHeight ) )
		{
	    	//IE 4 compatible
	    	iHeight = document.body.clientHeight;
		}
		return iHeight;
	}
	
	/*sfh
	******************************************************************************
	
		Function:		GetFilenameFromURL
	
		Desc:			Utility function for extracting the filename portion
						of a URL.
	
		Returns:		string
	
	******************************************************************************
	efh*/
	
	function GetFilenameFromURL(url)
	{
		var fn = "";
		var iPos;
		if (url)
		{
			iPos = url.lastIndexOf("/");
			iPos = Math.max(iPos, url.lastIndexOf("\\"));
	
			if (iPos != -1)
			{
				fn = url.substring(iPos, url.length);
			}
		}
		return fn;
	}
	
	/*sfh
	******************************************************************************
	
		Function:		CreateImage
	
		Desc:			Returns an Image object of the specified size.
	
		Returns:		Image
	
	******************************************************************************
	efh*/
	
	function CreateImage(width,height)
	{
		var img = document.createElement("img");
		if (width > 0 && height > 0)
		{
			img.width = width;
			img.height = height;
		}
		return img;
	}

	/*sfh
	******************************************************************************
	
		Function:		IsZoomSupported
	
		Desc:			Returns true if CSS zoom is supported by this browser.
	
		Returns:		boolean
	
	******************************************************************************
	efh*/
	
	function IsZoomSupported()
	{
		var zoom = document.body.style.zoom;
		return (typeof zoom != "undefined") ? true : false;
	}

}  // End of EngineObject

// End of file