/*sfh
******************************************************************************

		SERVER facing API: These functions are called by Athena server.

******************************************************************************
efh*/

function GetLocalAPIObject()
{
	if (!GetLocalAPIObject.api)
	{
		GetLocalAPIObject.api = new LocalAPIObject();
	}
	return GetLocalAPIObject.api;
}

//
// LocalAPIObject
//

function LocalAPIObject()
{
	//
	// Statics
	//
	this.SCODE_OK = 0;
	this.SCODE_INVALID_SESSIONID = -10;
	this.SCODE_INVALID_CLIENTID = -20;
	this.SCODE_INVALID_TILEID = -30;
	this.SCODE_INVALID_CONTENTKEY = -40;
	
	/*sfh
	******************************************************************************
	
		Function:		SetCallbackAPI
	
		Desc:			Sets the callback API, meaning the object that implements 
						the APIs called below. These methods are called via
						JavaScript downloaded from the Athena server.
						
						Generally the Appshviewer or Appshplayer object
						is passed here.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.SetCallbackAPI = function(callbacks)
	{
		Callbacks = callbacks;
	};

	// LocalAPIObject.ProcessCookies is defined below...

	/*sfh
	******************************************************************************
	
		Function:		LocalAPIObject.ProcessCookies
	
		Desc:			Provides an alternate way for the server to call local
						APIs - via cookies. 
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.ProcessCookies = function(cookieMgr)
	{
		var value;
		var params;
		
		// [SetCursorPos]  cpos=x,y
		value = cookieMgr.GetCookieWithDefault("cpos", null);
		if (value !== null)
		{
			params = value.split(",", 2);
			SetCursorPos(parseInt(params[0]), parseInt(params[1]));
		}
	
		// [SetScreenOffset]  soff=x,y
		value = cookieMgr.GetCookieWithDefault("soff", null);
		if (value !== null)
		{
			params = value.split(",", 2);
			SetScreenOffset(parseInt(params[0]), parseInt(params[1]));
		}
	
		// [ShowTooltip]  tip=bShow,text
		value = cookieMgr.GetCookieWithDefault("soff", null);
		if (value !== null)
		{
			params = value.split(",", 2);
			ShowTooltip((params[0]=="1" ? true : false), params[1]);
		}
	
		// [SetSharing]  sha=1
		value = cookieMgr.GetCookieWithDefault("sha", null);
		if (value !== null)
		{
			SetSharing(value=="1" ? true : false);
		}
		
		// [SetController]  cntr=1
		value = cookieMgr.GetCookieWithDefault("cntr", null);
		if (value !== null)
		{
			SetController(value=="1" ? true : false);
		}

		// [SetController] ALTERNATE SPELLING cnrt=1
		value = cookieMgr.GetCookieWithDefault("cnrt", null);
		if (value !== null)
		{
			SetController(value=="1" ? true : false);
		}

		// [SetSharedRect]  rect=left,top,width,height
		value = cookieMgr.GetCookieWithDefault("rect", null);
		if (value !== null)
		{
			params = value.split(",", 4);
			SetSharedRect(parseInt(params[0]), parseInt(params[1]), parseInt(params[2]), parseInt(params[3]));
		}
	
		// [EndSession]  endsess=1
		value = cookieMgr.GetCookieWithDefault("endsess", null);
		if (value !== null)
		{
			if (value=="1")
			{
				EndSession();
			}
		}
	
		// [CmdError]  err=-20
		value = cookieMgr.GetCookieWithDefault("err", null);
		if (value !== null)
		{
			CmdError("getnexttile", value, "Error returned by 'getnexttile' via cookie");
		}
	};

} // End of LocalAPIObject

//
// Objects
//
var Debug = GetDebugObject();
var API = GetLocalAPIObject();
var Callbacks = null;

/*sfh
******************************************************************************

	Function:		CmdComplete

	Desc:			Called by the server to indicate that a API call made 
					from the client to server has been completed.

	Returns:		void

******************************************************************************
efh*/

function CmdComplete(cmd, result)
{
	// Route the notification.
	if (cmd == "getupdate")
	{
		if (typeof Callbacks.OnCmdComplete_GetUpdate != "undefined")
		{	Callbacks.OnCmdComplete_GetUpdate(result);	}
	}
	else if (cmd == "joinsession")
	{
		if (typeof Callbacks.OnCmdComplete_JoinSession != "undefined")
		{	Callbacks.OnCmdComplete_JoinSession(result);	}
	}
	else if (cmd == "setimage")
	{
		if (typeof Callbacks.OnCmdComplete_SetImage != "undefined")
		{	Callbacks.OnCmdComplete_SetImage(result);	}
	}
	else if (cmd == "setcursor")
	{
		if (typeof Callbacks.OnCmdComplete_SetCursor != "undefined")
		{	Callbacks.OnCmdComplete_SetCursor(result);	}
	}
	else if (cmd == "createsession") 
	{
		if (typeof Callbacks.OnCmdComplete_CreateSession != "undefined")
		{	Callbacks.OnCmdComplete_CreateSession(result); }
	}
	else if (cmd == "updatecursor") 
	{
		if (typeof Callbacks.OnCmdComplete_UpdateCursor != "undefined")
		{	Callbacks.OnCmdComplete_UpdateCursor(result); }
	}
	else if (cmd == "updatekb")
	{
		if (typeof Callbacks.OnCmdComplete_UpdateKeystroke != "undefined")
		{	Callbacks.OnCmdComplete_UpdateKeystroke(result); }
	}
	else if (cmd == "updatecursorkb2")
	{
		if (typeof Callbacks.OnCmdComplete_UpdateCursorKb2 != "undefined")
		{	Callbacks.OnCmdComplete_UpdateCursorKb2(result); }
	}
	else
	{
		// Other notifications can be ignored.	
		Debug.ConsoleOut("CmdComplete notificaton - unknown command: " + cmd);
	}
}

/*sfh
******************************************************************************

	Function:		CmdError

	Desc:			Called by the server to indicate that a request has 
					resulted in an error. These errors are because of
					invalid parameters sent in the request eg. bad client id.

					This is nothing that the user can do about the error.

	Returns:		void

******************************************************************************
efh*/

function CmdError(cmd, result, description)
{
	Debug.ConsoleOut("ERROR FROM SERVER: cmd=" + cmd + "  ---  result=" + result + "  ---   descr=" + description);
	
	// Route the notification.
	if (cmd == "getupdate")
	{
		if (typeof Callbacks.OnCmdError_GetUpdate != "undefined")
		{	Callbacks.OnCmdError_GetUpdate(result, description);	}
	}
	else if (cmd== "getnexttile")
	{
		if (typeof Callbacks.OnCmdError_GetNextTile != "undefined")
		{	Callbacks.OnCmdError_GetNextTile(result, description);	}
	}
	else if (cmd == "joinsession")
	{
		if (typeof Callbacks.OnCmdError_JoinSession != "undefined")
		{	Callbacks.OnCmdError_JoinSession(result, description);	}
	}
	else if (cmd == "setimage")
	{
		if (typeof Callbacks.OnCmdError_SetImage != "undefined")
		{	Callbacks.OnCmdError_SetImage(result, description);	}
	}
	else if (cmd == "setcursor")
	{
		if (typeof Callbacks.OnCmdError_SetCursor != "undefined")
		{	Callbacks.OnCmdError_SetCursor(result, description);	}
	}
	else if (cmd == "createsession")
	{
		if (typeof Callbacks.OnCmdError_CreateSession != "undefined")
		{	Callbacks.OnCmdError_CreateSession(result, description);	}
	}
	else
	{
		// Other error notifications can be ignored.	
		Debug.ConsoleOut("OnError notificaton - unknown command: " + cmd);
	}
}

/*sfh
******************************************************************************

	Function:		SetClientID

	Desc:			Called by the server to indicate the client id to be used 
					by all future server calls that require client specific 
					context.

	Returns:		void

******************************************************************************
efh*/

function SetClientID(cid)
{
	if (typeof Callbacks.OnSetClientID != "undefined")
	{
		Callbacks.OnSetClientID(cid);	
	}
}

/*sfh
******************************************************************************

	Function:		SetSessType

	Desc:			Called when the appshare server sends the appshare type.

	Returns:		void

******************************************************************************
efh*/

function SetSessType(sesstype)
{
	if (typeof Callbacks.OnSetSessType != "undefined")
	{
		Callbacks.OnSetSessType(sesstype);	
	}
}

/*sfh
******************************************************************************

	Function:		SetFetchMode

	Desc:			Called when the appshare server sends the fetch mode.

	Returns:		void

******************************************************************************
efh*/

function SetFetchMode(newmode, tilecachesize, cursorcachesize)
{
	if (typeof Callbacks.OnSetFetchMode != "undefined")
	{
		Callbacks.OnSetFetchMode(newmode, tilecachesize, cursorcachesize);	
	}
}

/*sfh
******************************************************************************

	Function:		SetServerCk

	Desc:			Called by the server to indicate the content key to be used 
					by all API calls and tile fetches.

	Returns:		void

******************************************************************************
efh*/

function SetServerCk(ck)
{
	if (typeof Callbacks.OnSetServerCk != "undefined")
	{
		Callbacks.OnSetServerCk(ck);	
	}
}

/*sfh
******************************************************************************

	Function:		SetDimensions

	Desc:			Called by the server to indicate the size of the appshare
					screen has changed. The tile map must be rebuilt.

	Returns:		void

******************************************************************************
efh*/

function SetDimensions(numXTiles, numYTiles, tileWidth, tileHeight, screenWidth, screenHeight)
{ 
	if (typeof Callbacks.OnSetDimensions != "undefined")
	{
		Callbacks.OnSetDimensions(numXTiles, numYTiles, tileWidth, tileHeight, screenWidth, screenHeight);
	}
}

/*sfh
******************************************************************************

	Function:		SetDirtyTiles

	Desc:			Called by the server with a list of tile ids that need
					to have new images downloaded.

	Returns:		void

******************************************************************************
efh*/

function SetDirtyTiles(count, format)
{
	if (typeof Callbacks.OnSetDirtyTiles != "undefined")
	{
		Callbacks.OnSetDirtyTiles(count, format, SetDirtyTiles.arguments);
	}
}

/*sfh
******************************************************************************

	Function:		SetAppShareSessionID

	Desc:			Called by the server when the AppShare Session id changes.
					USED ONLY BY THE ATHENA SIMULATOR.

	Returns:		void

******************************************************************************
efh*/

function SetAppShareSessionID(strID)
{
	if (typeof Callbacks.OnSetAppShareSessionID != "undefined")
	{
		Callbacks.OnSetAppShareSessionID(strID);
	}
}

/*sfh
******************************************************************************

	Function:		SetCursorImage

	Desc:			Called by the server when the cursor image has changed.

	Returns:		void

******************************************************************************
efh*/

function SetCursorImageDirty(hexCRC, format)
{
	if (typeof Callbacks.OnSetCursorImageDirty != "undefined")
	{
		Callbacks.OnSetCursorImageDirty(hexCRC, format);
	}
}

/*sfh
******************************************************************************

	Function:		SetCursorPos

	Desc:			Called by the server when the cursor position has changed.

	Returns:		void

******************************************************************************
efh*/

function SetCursorPos(x,y) 
{  
	if (typeof Callbacks.OnSetCursorPos != "undefined")
	{
		Callbacks.OnSetCursorPos(x,y);
	}
}

/*sfh
******************************************************************************

	Function:		ShowTooltip

	Desc:			Turns on or off the display of the tooltip which follows
					the cursor around and displays the name of the active
					participant.

	Returns:		void

******************************************************************************
efh*/

function ShowTooltip(bShow, txt)
{
	if (typeof Callbacks.OnShowTooltip != "undefined")
	{
		Callbacks.OnShowTooltip(bShow, txt);
	}
}

/*sfh
******************************************************************************

	Function:		SetScreenOffset

	Desc:			Called by the server when the upper-left corner of the
					screen should be offset.

					DEPRECATED!

	Returns:		void

******************************************************************************
efh*/

function SetScreenOffset(x,y) 
{  
	if (typeof Callbacks.OnSetScreenOffset != "undefined")
	{
		Callbacks.OnSetScreenOffset(x,y);
	}
}

/*sfh
******************************************************************************

	Function:		SetSharedRect

	Desc:			Called by the server when the upper-left corner of the
					screen should be offset, or the width/height of the 
					shared area changes.

	Returns:		void

******************************************************************************
efh*/

function SetSharedRect(left, top, width, height) 
{  
	if (typeof Callbacks.OnSetSharedRect != "undefined")
	{
		Callbacks.OnSetSharedRect(left, top, width, height);
	}
}

/*sfh
******************************************************************************

	Function:		SetSharing

	Desc:			Called by the server when something is being shared, or
					when nothing is being shared. Aegis shows or hides the
					"No apps currently shared..." message.

	Returns:		void

******************************************************************************
efh*/

function SetSharing(bSharing) 
{  
	if (typeof Callbacks.OnSetSharing != "undefined")
	{
		Callbacks.OnSetSharing(bSharing);
	}
}

/*sfh
******************************************************************************

	Function:		SetController

	Desc:			Called by the server when this Aegis client has been 
					granted control or had it revoked.
					Being granted control means any mouse or keyboard events
					should be sent to the server, which are then pumped into
					the moderator's shared application.

	Returns:		void

******************************************************************************
efh*/

function SetController(bController) 
{  
	if (typeof Callbacks.OnSetController != "undefined")
	{
		Callbacks.OnSetController(bController);
	}
}

/*sfh
******************************************************************************

	Function:		DataAvailable

	Desc:			Called by the server when in fetch mode "stream" and there
					is image data available to be fetched via "getnexttile".

	Returns:		void

******************************************************************************
efh*/

function DataAvailable()
{
	if (typeof Callbacks.OnDataAvailable != "undefined")
	{
		Callbacks.OnDataAvailable();
	}
}

/*sfh
******************************************************************************

	Function:		EndSession

	Desc:			Called by the server when the AppShare Session has ended.

	Returns:		void

******************************************************************************
efh*/

function EndSession()
{
	if (typeof Callbacks.OnEndSession != "undefined")
	{
		Callbacks.OnEndSession();
	}
}

/*sfh
******************************************************************************

	Function:		AddScene

	Desc:			Adds a scene to the list when playing back archives.

	Returns:		void

******************************************************************************
efh*/

function AddScene(scenejs, scenejpg, starttime, endtime)
{ 
	if (typeof Callbacks.OnAddScene != "undefined")
	{
		Callbacks.OnAddScene(scenejs, scenejpg, starttime, endtime);
	}
}

/*sfh
******************************************************************************

	Function:		EndMaster

	Desc:			Called when the master.js file (list of all scenes) is 
				    finished processing.

	Returns:		void

******************************************************************************
efh*/

function EndMaster()
{ 
	if (typeof Callbacks.OnEndMaster != "undefined")
	{
		Callbacks.OnEndMaster();
	}
}

/*sfh
******************************************************************************

	Function:		EndScene

	Desc:			Called when a scene js file processing is completed.

	Returns:		void

******************************************************************************
efh*/

function EndScene()
{ 
	if (typeof Callbacks.OnEndScene != "undefined")
	{
		Callbacks.OnEndScene();
	}
}

/*sfh
******************************************************************************

	Function:		SetTile

	Desc:			Called during archive playback to set a tile to the image 
					within a specific rectangle in the scene PNG image.

	Returns:		void

******************************************************************************
efh*/

function SetTile(tid, pngX, pngY)
{
	//ConsoleOut("SetTile called, png path " + g_pngFilePath);
	if (typeof Callbacks.OnSetTile != "undefined")
	{
		Callbacks.OnSetTile( tid, pngX, pngY);
	}	
}

/*sfh
******************************************************************************

	Function:		SetMouseCursor

	Desc:			Called during archive playback to set the mouse cursor 
					to the image within a specific rectangle in the 
					cursors.gif image.

	Returns:		void

******************************************************************************
efh*/

function SetMouseCursor( gifX, gifY, cursorWidth, cursorHeight)
{
	//ConsoleOut("SetMouseCursor called");
	if (typeof Callbacks.OnSetCursor != "undefined")
	{
		Callbacks.OnSetCursor( gifX, gifY, cursorWidth, cursorHeight);
	}	
}

/*sfh
******************************************************************************

	Function:		SetKeyFrameTile

	Desc:			Called during archive playback, when seeking, to set a 
					tile to the image within a specific rectangle in the 
					specified scene PNG image. The image is the key frame
					image for the beginning of the scene.

	Returns:		void

******************************************************************************
efh*/

function SetKeyFrameTile(sid, tid, pngX, pngY)
{
	//ConsoleOut("SetKeyFrameTile called, png path " + g_pngFilePath);
	if (typeof Callbacks.OnSetKeyFrameTile != "undefined")
	{
		Callbacks.OnSetKeyFrameTile( sid, tid, pngX, pngY);
	}	
}

/*sfh
******************************************************************************

	Function:		EndKeyFrame

	Desc:			Called during archive playback, to indicate the end of
				    key frame data at the beginnging of every scene.

	Returns:		void

******************************************************************************
efh*/

function EndKeyFrame()
{
	//ConsoleOut("EndKeyFrame called");
	if (typeof Callbacks.OnEndKeyFrame != "undefined")
	{
		Callbacks.OnEndKeyFrame();
	}	
}

// End of file