/**sfh
******************************************************************************

		zoom
	~~~~~~~~~~~~~~
		 
		Provides methods for the ZOOM (scaling) feature. These are shared by
		the live and archive clients.


******************************************************************************
efh*/

//
// Objects
//
var Debug = GetDebugObject();
var D2E = GetDOM2EventObject();
var Comms = GetCommsObject();
var Engine = GetEngineObject();
var Zoom = GetZoomObject();

function GetZoomObject()
{
	if (!GetZoomObject.zoom)
	{
		GetZoomObject.zoom = new ZoomObject;
	}
	return GetZoomObject.zoom;
}

function ZoomObject()
{
	//
	// Vars
	//
	var m_localMouse = new Object();
	m_localMouse.pageX = 0;					// x and y position of local mouse cursor relative to document
	m_localMouse.pageY = 0;
	m_localMouse.clientX = 0;				// x and y position of local mouse cursor relative to browser client area
	m_localMouse.clientY = 0;

	/*sfh
	******************************************************************************
	
		Function:		CreateZoomUI
	
		Desc:			Creates the DIV containing UI that allows the user to 
						choose the zoom setting.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.CreateZoomUI = function()
	{
		// The UI is already in the XHTML page, so we just need
		// to set styles, add handlers and text.
		var divZoomUI = document.getElementById("divZOOM_UI");
		divZoomUI.style.zIndex = "100000";
		divZoomUI.style.zoom = "100%";
		divZoomUI.onmouseup = Zoom.On_ZoomDivMouseUp;
		divZoomUI.onmousemove = Zoom.On_ZoomDivMouseMove;
	
		var imgMagMinus = document.getElementById("imgMagMinus");
		imgMagMinus.style.zIndex = "100001";
		imgMagMinus.onclick = Zoom.On_MagMinusButtonClicked;

		var combo = document.getElementById("comboZoom");
		combo.style.zIndex = "100002";
		combo.onchange = Zoom.On_ZoomComboChange;

		var imgMagPlus = document.getElementById("imgMagPlus");
		imgMagPlus.style.zIndex = "100003";
		imgMagPlus.onclick = Zoom.On_MagPlusButtonClicked;

		// Hide it
		Zoom.ShowZoomUI(false);
		
		// Set up the ZOOM tooltip
		var tooltip = document.getElementById("divZOOM_Tooltip");
		tooltip.style.position = "absolute";
		tooltip.style.backgroundColor = "#FFFFE1"; // use the same color as the other appsharing tooltip
		tooltip.style.border = "1px solid black";
		tooltip.style.color = "black";
		tooltip.style.font = "normal normal normal 7pt sans-serif";
	    tooltip.style.marginTop = "0px";    // location below the cursor image
		tooltip.style.paddingTop = "0px";
		tooltip.style.paddingBottom = "0px";
		tooltip.style.paddingLeft = "2px";
		tooltip.style.paddingRight = "2px";
		tooltip.style.zIndex = "100001";
		tooltip.style.zoom = "100%";	
		tooltip.style.visibility = "hidden";

		// Localize the "Auto-Fit" and "Fit Width" option strings
		LocalizeZoomStrings();
	};

	/*sfh
	******************************************************************************
	
		Function:		LocalizeZoomStrings
	
		Desc:			Updates the text in the zoom UI based on language 
						settings.
	
		Returns:		void
	
	******************************************************************************
	efh*/

	function LocalizeZoomStrings()
	{
		if (!Comms.FetchStrings.bFetchComplete)
		{
			// String table is not fetched yet, try again later.
			return;
		}
		
		if (Comms.FetchStrings.stringtable[Comms.ATH_ZOOM_AUTO])
		{
			var optionAuto = document.getElementById("optionAuto");
			optionAuto.text = Comms.FetchStrings.stringtable[Comms.ATH_ZOOM_AUTO];
		} 
		if (Comms.FetchStrings.stringtable[Comms.ATH_ZOOM_WIDTH])
		{
			var optionWidth = document.getElementById("optionWidth");
			if (optionWidth) { optionWidth.text = Comms.FetchStrings.stringtable[Comms.ATH_ZOOM_WIDTH]; }
		} 

		var zoomTooltip = document.getElementById("divZOOM_Tooltip");
		if (Comms.FetchStrings.stringtable[Comms.ATH_ZOOM_TOOLTIP])
		{
			if (zoomTooltip) { zoomTooltip.innerHTML = Comms.FetchStrings.stringtable[Comms.ATH_ZOOM_TOOLTIP]; }
		}
		else
		{
			if (zoomTooltip) { zoomTooltip.innerHTML = "Click to zoom."; }
		} 
		LocalizeZoomStrings.bDone = true;
	}

	/*sfh
	******************************************************************************
	
		Function:		ShowZoomUI
	
		Desc:			Shows or hides the Zoom UI.
	
		Returns:		void
	
	******************************************************************************
	efh*/

	this.ShowZoomUI = function(bShow)
	{
		// Cancel the UI timer if nessesary
		if (Zoom.UpdateZoomUIVisibility.timer)
		{
			window.clearTimeout(Zoom.UpdateZoomUIVisibility.timer);
			Zoom.UpdateZoomUIVisibility.timer = 0;
		}

		// Hide the zoom tooltip.
		Zoom.ShowZoomTooltip(false);
		
		var divZoomUI = document.getElementById("divZOOM_UI");
		var combo = document.getElementById("comboZoom");
		var imgMagMinus = document.getElementById("imgMagMinus");
		var imgMagPlus = document.getElementById("imgMagPlus");

		var txtVisible = "hidden";

		// First we need to position the zoom UI div in the right place
		if (bShow)
		{
			Debug.ConsoleOut("ShowZoomUI: clientX=" + m_localMouse.clientX + "; pageX=" + m_localMouse.pageX);

			var x = Math.max(1, Math.min(m_localMouse.pageX, Engine.EngGetPageWidth() - divZoomUI.offsetWidth - 5));
			var y = Math.max(1, Math.min(m_localMouse.pageY, Engine.EngGetPageHeight() - divZoomUI.offsetHeight - 5));
			
			divZoomUI.style.left = x;
			divZoomUI.style.top = y;
			
			divZoomUI.onmouseup = Zoom.On_ZoomDivMouseUp;
			
			UpdateZoomCombo();
			
			txtVisible = "visible";
			Zoom.UpdateZoomUIVisibility.timer = window.setTimeout("Zoom.UpdateZoomUIVisibility();", 5000);
		}

		divZoomUI.style.visibility = txtVisible;
		combo.style.visibility = txtVisible;
		imgMagMinus.style.visibility = txtVisible;
		imgMagPlus.style.visibility = txtVisible;
	};

	/*sfh
	******************************************************************************
	
		Function:		On_MouseMove
	
		Desc:			Called whenver the local mouse cursor is moved across the
						tile div. We need to keep track of mouse position and 
						last move time for turning off UI such as zoom UI.
	
		Returns:		void
	
	******************************************************************************
	efh*/

	this.On_MouseMove = function(evt)
	{
		var event = D2E.CreateDOM2Event(evt, window.event);
		var bChanged = (m_localMouse.pageX != event.pageX || m_localMouse.pageY != event.pageY) ? true : false;
		
		m_localMouse.pageX = event.pageX;
		m_localMouse.pageY = event.pageY;
		m_localMouse.clientX = event.clientX;
		m_localMouse.clientY = event.clientY;

		if (bChanged)
		{		
			// Show the tooltip only if the zoom UI is not displayed. 
			Zoom.ShowZoomTooltip(!IsZoomUIVisible());
		}
	};

	/*sfh
	******************************************************************************
	
		Function:		On_MouseUp
	
		Desc:			Called when a mouseup event is received by the document.
	
		Returns:		void
	
	******************************************************************************
	efh*/

	this.On_MouseUp = function(evt)
	{
		// If the zoom UI is hidden, then show it, otherwise hide it.
		var event = D2E.CreateDOM2Event(evt, window.event);
		Zoom.ShowZoomUI(!IsZoomUIVisible());
		D2E.CancelBubbling(event);
	};

	/*sfh
	******************************************************************************
	
		Function:		On_ZoomDivMouseUp
	
		Desc:			Called when a mouseup event is received by the 
						zoom ui div.
	
		Returns:		void
	
	******************************************************************************
	efh*/

	this.On_ZoomDivMouseUp = function(evt)
	{
		var event = D2E.CreateDOM2Event(evt, window.event);
		D2E.CancelBubbling(event);
	};

	/*sfh
	******************************************************************************
	
		Function:		On_ZoomDivMouseMouse
	
		Desc:			Called when a mousemouse event is received by the 
						zoom ui div.
	
		Returns:		void
	
	******************************************************************************
	efh*/

	this.On_ZoomDivMouseMove = function(evt)
	{
		Zoom.On_MouseMove(evt);
		var event = D2E.CreateDOM2Event(evt, window.event);
		D2E.CancelBubbling(event);
		Zoom.UpdateZoomUIVisibility();
	};

	/*sfh
	******************************************************************************
	
		Function:		IsZoomUIVisible
	
		Desc:			Returns true if the zoom UI div is visible.
	
		Returns:		boolean
	
	******************************************************************************
	efh*/
	
	function IsZoomUIVisible()
	{
		var divZoomUI = document.getElementById("divZOOM_UI");
		return (divZoomUI.style.visibility == "hidden") ? false : true;
	}

	/*sfh
	******************************************************************************
	
		Function:		UpdateZoomUIVisibility
	
		Desc:			Hides the ZoomUI when the mouse is outside the ZoomUI div
						and has not moved for 5 seconds.
	
		Returns:		void
	
	******************************************************************************
	efh*/
	
	this.UpdateZoomUIVisibility = function()
	{
		if (Zoom.UpdateZoomUIVisibility.timer)
		{
			window.clearTimeout(Zoom.UpdateZoomUIVisibility.timer);
			Zoom.UpdateZoomUIVisibility.timer = 0;
		}

		// Check if the local mouse cursor is inside the zoom UI div
		var divZoomUI = document.getElementById("divZOOM_UI");
		var left = parseInt(divZoomUI.style.left);
		var top = parseInt(divZoomUI.style.top);

		if (m_localMouse.pageX >= left && m_localMouse.pageY >= top && 
			m_localMouse.pageX <= left + divZoomUI.offsetWidth &&
			m_localMouse.pageY <= top + divZoomUI.offsetHeight)
		{
			Zoom.UpdateZoomUIVisibility.timer = window.setTimeout("Zoom.UpdateZoomUIVisibility();", 5000);
		}
		else
		{
			Zoom.ShowZoomUI(false);		
		}
	};

	/*sfh
	******************************************************************************
	
		Function:		ShowZoomTooltip
	
		Desc:			Shows or hides the zoom tooltip. When shown, a timer
						is created to hide it again after several seconds.
	
		Returns:		void
	
	******************************************************************************
	efh*/

	this.ShowZoomTooltip = function(bShow)
	{
		// Cancel the timer.
		if (Zoom.ShowZoomTooltip.timer)
		{
			window.clearTimeout(Zoom.ShowZoomTooltip.timer);
			Zoom.ShowZoomTooltip.timer = 0;
		}
		
		var tooltip = document.getElementById("divZOOM_Tooltip");
		var x = Math.max(1, Math.min(m_localMouse.pageX, Engine.EngGetPageWidth() - tooltip.offsetWidth - 5));
		var y = Math.max(1, Math.min(m_localMouse.pageY + 40, Engine.EngGetPageHeight() - tooltip.offsetHeight - 5));
			
		tooltip.style.left = x;
		tooltip.style.top = y;
		tooltip.style.visibility = bShow ? "visible" : "hidden";

		if (bShow)
		{
			// Unfortunately we have to keep updating the tooltip text because the
			// language strings are loaded async and we don't know when that is
			// finished.
			if (!LocalizeZoomStrings.bDone) { LocalizeZoomStrings(); }

			Zoom.ShowZoomTooltip.timer = window.setTimeout("Zoom.ShowZoomTooltip(false);", 1500);
		}
	};

	/*sfh
	******************************************************************************
	
		Function:		On_MagMinusButtonClicked
	
		Desc:			Called when the minus magnification icon is clicked.
						Decrement the zoom factor by 25%.
	
		Returns:		void
	
	******************************************************************************
	efh*/

	this.On_MagMinusButtonClicked = function(evt)
	{
		var iZoom = Engine.EngGetActualZoom();
		// Round to the nearest 5
		iZoom = Math.round(iZoom/5) * 5;

		Engine.EngSetZoom(iZoom - 5);
		
		var event = D2E.CreateDOM2Event(evt, window.event);
		D2E.CancelBubbling(event);
		
		UpdateZoomCombo();
	};

	/*sfh
	******************************************************************************
	
		Function:		On_ZoomComboChange
	
		Desc:			Called when the zoom level selector has been changed by 
						the user.
							
		Returns:		void
	
	******************************************************************************
	efh*/

	this.On_ZoomComboChange = function(evt)
	{
		var combo = document.getElementById("comboZoom");

		Engine.EngSetZoom(combo.value);
		UpdateZoomCombo();
	};

	/*sfh
	******************************************************************************
	
		Function:		On_MagPlusButtonClicked
	
		Desc:			Called when the plus magnification icon is clicked.
						Increment the zoom factor by 25%.
	
		Returns:		void
	
	******************************************************************************
	efh*/

	this.On_MagPlusButtonClicked = function(evt)
	{
		var iZoom = Engine.EngGetActualZoom();
		// Round to the nearest 5
		iZoom = Math.round(iZoom/5) * 5;

		Engine.EngSetZoom(iZoom + 5);
		
		var event = D2E.CreateDOM2Event(evt, window.event);
		D2E.CancelBubbling(event);

		UpdateZoomCombo();
	};

	/*sfh
	******************************************************************************
	
		Function:		UpdateZoomCombo
	
		Desc:			Called periodically to update the zoom ui combo box
						so that the first option item reflects the current
						zoom level.
	
		Returns:		void
	
	******************************************************************************
	efh*/

	function UpdateZoomCombo()
	{
		// Unfortunately we have to keep updating the combo text because the
		// language strings are loaded async and we don't know when that is
		// finished.
		if (!LocalizeZoomStrings.bDone) { LocalizeZoomStrings(); }
		
		var optCurrent = document.getElementById("optionCurrent");
		var optAuto = document.getElementById("optionAuto");
		var optWidth = document.getElementById("optionWidth");
		var iZoom = Engine.EngGetActualZoom();
		var strZoom = Engine.EngGetZoom();

		optCurrent.text	= iZoom + "%";
		optCurrent.value = iZoom;

		// Now select the correct item.
		if (strZoom == "auto" && optAuto)
		{
			optAuto.selected = true;
		}
		else if (strZoom == "width" && optWidth)
		{
			optWidth.selected = true;
		}
		else if (optCurrent)
		{
			optCurrent.selected = true;
		}
	}

} // End of ZoomObject

// End of file